from esgcet.pub_client import publisherClient
import esgcet.logger as logger
import os


from esgcet.globus_search import GlobusSearchIngest

log = logger.ESGPubLogger()


class ESGPubIndex:
    """
    Wrapper class for push-publishing of records to the index node.
    """
    def __init__(self, index_node="", UUID=None,  verbose=False, silent=False, verify=True, auth=None, arch_cfg=None, file_cache=None, dry_run=False):
        """
        Constructor, creates a "client" object
        """
        self.silent = silent
        self.verbose = verbose
        if index_node:
            self.pubCli = publisherClient("", index_node, verify=verify, verbose=self.verbose, silent=self.silent, auth=auth)
        else:
            self._index_UUID = UUID
        self.publog = log.return_logger('Index Publication', silent, verbose)
        self.arch_cfg = arch_cfg

        
        self._cache_dir = file_cache
        self._dry_run = dry_run

    def do_globus(self, d):
        gs = GlobusSearchIngest(d, self._cache_dir)
        res = gs.run(False)
        if not self._dry_run:
            gs.extern_globus_publish(res, self._index_UUID)
        else:
            self.publog.info(f"dry run on, not publishing {res}")
#        os.system(f"globus search ingest {self._index_UUID} {res}")

    def gen_xml(self, d):
        out = []
        out.append("<doc>\n")

        for key in d:

            val = d[key]
            if key == "description":
                val = ' '.join(val)
                out.append('  <field name="{}">{}</field>\n'.format(key, val))
            elif type(val) is list:
                for vv in val:
                    out.append('  <field name="{}">{}</field>\n'.format(key, vv))
            else:
                out.append('  <field name="{}">{}</field>\n'.format(key, val))
        out.append("</doc>\n")
        return ''.join(out)

    def do_publish(self, dataset):
        """ handle dataset publishing
        dataset (list) of dictionary records
        """
        rc = True
        for rec in dataset:

            new_xml = self.gen_xml(rec)

            if self.arch_cfg:
                resp = self.archive_rec(rec, new_xml)
                if not resp:
                    self.publog.error("Error in archiving.  Archiving haulting.  Publication will continue")
                    self.arch_cfg = None
                    rc = False
            self.publog.debug(new_xml)
            self.pubCli.publish(new_xml)
        return rc

    def pub_xml(self, doc):
        self.pubCli.publish(doc)

    def archive_rec(self, rec, new_xml):

        if "dataset_id" in rec:
            id_key = "dataset_id"
        else:
            id_key = "id"
        pathid = rec[id_key]
        pathid = pathid[:pathid.find('|')]
        dsparts = pathid.split('.')
        fname = rec["id"] + ".xml"

        pathlen = self.arch_cfg["length"] 
        if pathlen  > len(dsparts):
            pathlen = len(dsparts)
        subpath = '/'.join(dsparts[0:pathlen])


        destpath = os.path.join(self.arch_cfg["archive_path"], subpath)
        try:
            res = os.system(f"mkdir -p {destpath}")
            if not res == 0:
                raise RuntimeError(f"{destpath} {res} (possible permission error?)")
        except Exception as ex:
            self.publog.exception(f"Error creating archive directory {ex}")
            exit(1)
        try:
            with open(os.path.join(destpath, fname), "w") as outf:
                outf.write(new_xml)
        except Exception as ex:
            self.publog.exception(f"Error writing xml file to archive. {ex}")
            return False
        return True



import xarray, netCDF4
from esgcet.handler_base import ESGPubHandlerBase
import os.path
import numpy as np

class ESGPubXArrayHandler(ESGPubHandlerBase):

    @staticmethod
    def xarray_load(map_data):
        datafile = map_data[0][1]
        destpath = os.path.dirname(datafile)

        filespec = f"{destpath}/*.nc"
        res = xarray.open_mfdataset(filespec, use_cftime=True)
        return res

    def get_attrs_dict(self, scanobj):
        return scanobj.attrs

    def get_scanfile_dict(self, scandata, mapdata):
        ret = {}
        for rec in mapdata:
            fn = rec['file']
            ds = netCDF4.Dataset(fn)
            try:
                ret[fn] = {"tracking_id": ds.tracking_id}
            except:
                self.publog.warn("Tracking ID not found")
                ret[fn] = {}
        return ret
    
    def get_variables(self, scanobj):
        res = {}
        for x in scanobj.variables:
            res[x] = scanobj.variables[x].attrs
        return res
    
    def get_variable_list(self, variable):
        return [x for x in variable]

    def _get_time_str(self, timeval):
        if type(timeval.item()) is int:
            x = str(timeval)
            idx = x.index('.')
            return x[:idx] + 'Z'
        else:
            return timeval.item().isoformat(timespec="seconds") + "Z"
        
    def _get_min_max_bounds(self, latlon):
        bigarr = latlon[0] + latlon[-1]
        return float(np.min(bigarr)), float(np.max(bigarr))
    
    def set_bounds(self, record, scanobj):

        geo_units = []
        # latitude
        if "lat" in scanobj.coords:
            lat = scanobj.coords["lat"]
            if len(lat) > 0:
                record["north_degrees"] = lat.values.max()
                record["south_degrees"] = lat.values.min()
            else:
                self.publog.warn("'lat' found but len 0")          
        elif "latitude" in scanobj.coords:
            lat = scanobj.coords["latitude"]
            if len(lat) > 0:
                if isinstance(lat[0].values, (list, np.ndarray)):
                    min, max = self._get_min_max_bounds(lat)
                    record["north_degrees"] = min
                    record["south_degrees"] = max

                else:    
                    record["north_degrees"] = lat[-1].values.item()
                    record["south_degrees"] = lat[0].values.item()
                geo_units.append(lat.units)
            else:
                self.publog.warn("Latitude found but len 0")
        else:
            self.publog.warn("Lat/Latitude not found")
        # longitude
        if "lon" in scanobj.coords:
            lon = scanobj.coords["lon"]
            if len(lon) > 0:
                record["east_degrees"] = lon.values.max()
                record["west_degrees"] = lon.values.min()
            else:
                self.publog.warn("'lon' found but len 0")          
        elif "longitude" in scanobj.coords:
            lon = scanobj.coords["longitude"]
            if len(lon) > 0:
                if isinstance(lon[0].values, (list, np.ndarray)):
                    min, max = self._get_min_max_bounds(lon)   
                    record["east_degrees"] = min
                    record["west_degrees"] = max
                else:
                    record["east_degrees"] = lon[-1].values.item()
                    record["west_degrees"] = lon[0].values.item()
                geo_units.append(lon.units)
            else:
                self.publog.warn("Latitude found but len 0")
                # time
        else:
            self.publog.warn("Lon/Longitude not found")
        if "time" in scanobj.coords:
            ti = scanobj.coords["time"]
            record["datetime_start"] = self._get_time_str(ti[0].values)
            record["datetime_end"] = self._get_time_str(ti[-1].values)
        # plev
        if "plev" in scanobj.coords:
            try:
                plev = scanobj.coords["plev"]
                record["height_top"] = plev[0].values.item() 
                record["height_bottom"] = plev[-1].values.item() 
                geo_units.append(plev.units)
            except:
                self.publog.warn("plev found but not an expected type")
        if len(geo_units) > 0:
            record["geo_units"] = geo_units
            



import requests
import esgcet.logger as logger
from esgcet.xmlfix import fixup_xml

log = logger.ESGPubLogger()

class publisherClient(object):
    """  REST API wrapper for the esg-seach/ws APIs
        User must supply API parameters in xml form for publish and updates, dataset_ids for retract/delete
    """
    
    def __init__(self, cert_fn, hostname, verify=False, verbose=False, silent=False, auth=False):
        """ cert_fn - Path to certificate file
            hostame - target index node for API
            verbose, silent (bool) - add additional output, suppress INFO messages
        """ 
        self.certFile = cert_fn
        self.keyFile = cert_fn

        urlbase = 'https://{}/esg-search/ws'.format(hostname)

        self.retractUrl = '{}/retract'.format(urlbase)
        self.updateUrl = '{}/update'.format(urlbase)
        self.publishUrl = '{}/publish'.format(urlbase)
        self.deleteUrl = '{}/delete'.format(urlbase)
        self.verbose = verbose
        self.silent = silent
        self.verify = verify
        self.use_cert = auth
        self.publog = log.return_logger('Publisher Client', silent, verbose)

    def post_data(self, url, data):
        """ Internal method to post data to a url via requests
            url - the url
            data - the post data payload
        """
        new_data = data #fixup_xml(data) #TODO: Re-Include
        if self.verbose and data != new_data:
            self.publog.info(f'payload before tweaking XML: {data}')
            self.publog.info(f'payload after tweaking XML: {new_data}')
        data = new_data
        
        if self.use_cert:
            resp = requests.post(url, data=data, cert=(self.certFile, self.keyFile), verify=self.verify, allow_redirects=True)
        else:
            resp = requests.post(url, data=data, verify=self.verify, allow_redirects=True)
        self.publog.info(resp.text)
        if resp.status_code != 200:
            self.publog.error("code = {}".format(resp.status_code))
            exit(-1 * resp.status_code)
        return resp
    
    def publish(self, xmldata):
        """  Invoke the publish API call
            xmldata - xml publication record to post
        """
        try:
            response = self.post_data(self.publishUrl, xmldata)
            if response.status_code != 200:
                raise RuntimeError('Publish returned status code' + str(response.status_code))
        except requests.exceptions.SSLError as e:
            self.publog.exception("SSL error!")
        except Exception as e:
            self.publog.exception("Some other error!")

    def update(self, xmldata):
        """  Invoke the update API call
            xmldata - xml update record to post
        """

        try:
            response = self.post_data(self.updateUrl, xmldata)
            if response.status_code != 200:
                raise RuntimeError('Update returned status code' + str(response.status_code))

        except requests.exceptions.SSLError as e:
            self.publog.exception("SSL error!")
        except Exception as e:
            self.publog.exception("Some other error!")

    def retract(self, object_id):
        """  Invoke the retract API call
            object_id - name of dataset id to retract in master_id.version|data_node form. 
        """
        
        data = { 'id' : object_id }
        self.publog.debug(data)

        try:
            response = self.post_data(self.retractUrl, data)
        except requests.exceptions.SSLError as e:
            self.publog.exception("SSL error!")
        except Exception as e:
            self.publog.exception("Some other error!")
        # root = etree.fromstring(response.content)
        # text = root[0].text
        # return (response.status_code, text)

    def delete(self, object_id):
        """  Invoke the delete API call
            object_id - name of dataset id to delete in master_id.version|data_node form. 
        """

        data = { 'id' : object_id }
        self.publog.debug(data)
        try:
            response = self.post_data(self.deleteUrl, data)
        except requests.exceptions.SSLError as e:
            self.publog.exception("SSL error!")
        except Exception as e:
            self.publog.exception("Some other error!")



from esgcet.mkd_input4mips import ESGPubMKDinput4MIPs
from esgcet.pid_cite_pub import ESGPubPidCite
from esgcet.cmip6 import cmip6
import esgcet.logger as logger

log = logger.ESGPubLogger()

class input4mips(cmip6):

    def __init__(self, argdict):
        super().__init__(argdict)
        self.MKD_Construct = ESGPubMKDinput4MIPs
        self.publog = log.return_logger('input4MIPs', self.silent, self.verbose)

    def pid(self, out_json_data):

        pid = ESGPubPidCite(out_json_data, self.pid_creds, self.data_node, test=self.test, silent=self.silent, verbose=self.verbose, disable_cite=self._disable_citation)
        try:
            new_json_data = pid.do_pidcite()
        except Exception as ex:
            self.publog.exception("Failed to assign pid")
            self.cleanup()
            exit(1)
        return new_json_data


    def workflow(self):

        # step one: convert mapfile
        self.publog.info("Converting mapfile...")
        map_json_data = self.mapfile()

        # step three: extract data
        self.publog.info(f"Running Extraction... {str(self.extract_method)}")
        self.extract_method(map_json_data)

        # step four: make dataset
        self.publog.info("Making dataset...")
        out_json_data = self.mk_dataset(map_json_data)

        # step five: assign PID
        self.publog.info("Assigning PID...")
        new_json_data = self.pid(out_json_data)

        # step seven: publish to database
        self.publog.info("Running index pub...")
        rc = self.index_pub(new_json_data)

        self.publog.info("Done. Cleaning up.")
        self.cleanup()
        return rc



import sys, json
import os
from esgcet.pid_cite_pub import ESGPubPidCite
from esgcet.search_check import ESGSearchCheck
from esgcet.pub_client import publisherClient

import esgcet.logger as logger

log = logger.ESGPubLogger()

import pdb

class ESGUnpublishSolr:

    def __init__(self):
        self.publog = log.return_logger('esgunpublish-2')
    
    def check_for_pid_proj(self, dset_arr):
    
        for dset in dset_arr:
             parts = dset.split('.')
             if parts[0].lower() in ["cmip6", "input4mips"]:
                 return True
        return False
    
    def run(self, args):

        self._args = args
        hostname = args["index_node"]
        data_node = args["data_node"]
        verbose = args["verbose"]    
        silent = args["silent"]
        auth = args["auth"]
        cert_fn = args["cert"]
        do_delete = args["delete"]

        searchcheck = ESGSearchCheck(hostname, silent, verbose)

        pub_log = log.return_logger('Unpublish', args["silent"], args["verbose"])
        status = 0

        for dset_id in args["dataset_id_lst"]:
    
            status += self.single_unpublish(dset_id, pub_log, searchcheck)
        return status
    
    def single_unpublish(self, dset_id, pub_log, searchcheck):

        args = self._args
        
        hostname = args["index_node"]
        do_delete = args["delete"]
        data_node = args["data_node"]
        cert_fn = args["cert"]
        auth = args["auth"]

        second_split = []
        if '|' in dset_id:
            first_split = dset_id.split('|')
            second_split = first_split[0].split('.')
            data_node = first_split[1]
        else:
            second_split = dset_id.split('.')
            dset_id_new = '{}|{}'.format(dset_id, data_node)
            dset_id = dset_id_new

        found, notretracted = searchcheck.run_check(dset_id)
    
        if not found:
            return(-1)
        
        if (not notretracted) and (not do_delete):
            pub_log.info("Use --delete to permanently erase the retracted record")
            return(0)
        
        if "pid_creds" in args and self.check_for_pid_proj([dset_id]):
            version = second_split[-1][1:]
            master_id = '.'.join(second_split[0:-1])
            pid_module = ESGPubPidCite({}, args["pid_creds"], data_node, False, args["silent"], args["verbose"])
            ret = pid_module.pid_unpublish(master_id, version)
            if not ret:
                pub_log.warning("PID Module did not return success")
        # ensure that dataset id is in correct format, use the set data node as a default
    
    
        
        pubCli = publisherClient(cert_fn, hostname, auth=auth, verbose=args["verbose"], silent=args["silent"])

        if do_delete:
            pubCli.delete(dset_id)
        else:
            pubCli.retract(dset_id)
        return(0)



DEBUG = False

INDEX_NODE = "esgf-node.llnl.gov"
DATA_NODE = None

CMOR_PATH = None
AUTOC_PATH = None
# these settings are presently managed within esg.ini and esg.<project>.ini

# Project-specific settings
#TODO: DRS and GA should be managed by a remote project service.  
#    Then, set by the workflow at reasonable intervals

# For each project these become the . delimited components of the dataset_id
DRS = { 'cmip6' : [ 'mip_era' , 'activity_drs','institution_id','source_id','experiment_id','member_id','table_id','variable_id','grid_label'],
         'e3sm' : [ 'source', 'model_version', 'experiment', 'grid_resolution', 'realm', 'regridding', 'data_type', 'time_frequency', 'ensemble_member'],
    'input4mips' : ['activity_id', 'mip_era', 'target_mip', 'institution_id', 'source_id', 'realm', 'frequency', 'variable_id', 'grid_label'],
    'cordex' : ['project', 'product', 'domain', 'institute', 'driving_model', 'experiment', 'ensemble', 'rcm_model', 'rcm_version', 'time_frequency', 'variable' ],
    'create-ip-exp' : ['project', 'product', 'institute', 'experiment', 'realm', 'time_frequency'],
    'create-ip-src': ['project', 'product', 'institute', 'source_id', 'realm', 'time_frequency'],
    'create-ip-model': ['project', 'product', 'institute', 'model', 'source_id', 'realm', 'time_frequency'],
    'create-ip-var': ['project', 'product', 'institute', 'model', 'source_id', 'time_frequency', 'realm', 'variable'],
        'cmip5' : ['project', 'product', 'institute', 'model', 'experiment', 'realm', 'time_frequency',  'ensemble'],
       'obs4mips' : ["activity_id", "institution_id", "source_id", "frequency", "variable_id", "grid_label"] }

SPLIT_FACET = { 'e3sm' : { 'delim' : '_' , 'facet' : 'grid_resolution', 0 : ''}  }

# Global attributes expected to be read for a particular project.  For now a simple list.  
GA = { 'cmip6' : ['frequency',
                     'realm',
                     'product',
                     'nominal_resolution',
                     'source_type',
                     'grid',
                     'creation_date',
                     'variant_label',
                     'sub_experiment_id',
                     'further_info_url',
                     'activity_id',
                     'data_specs_version', 'title'],
       'input4mips' : ['contact',
'dataset_category',
'source_version',
'source',
'further_info_url',
'title',
'product',
'nominal_resolution',
'deprecated',
'dataset_status',
'Conventions',
'target_mip_list',                       
'creation_date'],
"obs4mips" : ["realm", "product", "nominal_resolution", "source_type", "creation_date", "institution", "source",  "source_type", "contact", "region", "data_specs_version",  "further_info_url", "source_version_number"]
}

GA_DELIMITED = { 'cmip6' : { 'source_type' : ' ', 'activity_id' : ' ', 'realm' : ' '  }}
#                 'input4mips' : {'target_mip_list' : ','}}

CONST_ATTR =  { 'cmip6' : { 'model_cohort' : 'Registered', 'project' : 'CMIP6' },
              'obs4mips' : {'project' :'obs4MIPs'}, 'input4mips' : { 'project' :'input4MIPs' }
              }

GA_MAPPED = { 'cmip6' : { 'experiment' : 'experiment_title'} }

# the prefix is found in the published urls that are backed by the path prefix below
DATA_ROOTS = { }


SOURCE_ID_LIMITS = { 'cmip6' : 25}

# a certificate file for the index, assumes in the CWD
CERT_FN = "/p/user_pub/publish-queue/certs/certificate-file"

# for these the following are inserted in order: 1. hostname 2. prefix 3. relative dataset path
# Eg replace /thredds/fileServer with the prefix for NginX
# Note these are netCDF specific and will need to change if other formats are considered
URL_Templates = ["https://{}/thredds/fileServer/{}/{}|application/netcdf|HTTPServer",
"https://{}/thredds/dodsC/{}/{}|application/opendap-html|OPENDAP",
                 "globus:{}/{}/{}|Globus|Globus"]

DATASET_GLOBUS_URL_TEMPLATE = "https://app.globus.org/file_manager?origin_id={}&amp;origin_path=/{}"


#        handle-esgf-trusted.dkrz.de | 5671 | esgf-pid | esgf-publisher 

PID_CREDS = [ {'url': 'aims4.llnl.gov',
             'port': 7070,
             'vhost': 'esgf-pid',
             'user': 'esgf-publisher',
             'password': "",
             'ssl_enabled': True,
             'priority': 1}] 

PID_PREFIX = '21.14100' # for testing use CMIP6,  need to be project-specific
PID_EXCHANGE = 'esgffed-exchange'
HTTP_SERVICE = '/thredds/fileServer/'

CITATION_URLS = { 'cmip6' : {'test' :
'http://cera-www.dkrz.de/WDCC/testmeta/CMIP6/{}.v{}.json' ,
        'prod' : 'http://cera-www.dkrz.de/WDCC/meta/CMIP6/{}.v{}.json'},
'input4mips' : {'test' :
'http://cera-www.dkrz.de/WDCC/testmeta/CMIP6/{}.v{}.json' ,
        'prod' : 'http://cera-www.dkrz.de/WDCC/meta/CMIP6/{}.v{}.json'}
                  
}

PID_URL = 'http://hdl.handle.net/{}|PID|pid'  # PIDs include hdl:
TEST_PUB = True

PROJECT = "input4MIPs"  # project setting.  This would be used to consider some project-specific features, eg. for CMIP6
SET_REPLICA = False


VARIABLE_LIMIT = 75

VARIABLE_EXCLUDES = [ "lat_bounds", "lon_bounds", "time_bounds"]

CACHE_DIR_DEPTH = 6




import mapfile as mp
import mk_dataset as mkd
import update as up
import pub_test as pt
import pid_cite_pub as pid
import os
import json
import sys
import tempfile
import subprocess
from cmip6_cv import PrePARE
import args
import timeit


def prepare_internal(json_map, cmor_tables):
    print("iterating through filenames for PrePARE (internal version)...")
    validator = PrePARE.PrePARE
    for info in json_map:
        filename = info[1]
        process = validator.checkCMIP6(cmor_tables)
        process.ControlVocab(filename)


def check_files(files):
    for file in files:
        try:
            myfile = open(file, 'r')
        except Exception as ex:
            print("Error opening file " + file + ": " + str(ex))
            exit(1)
        myfile.close()


def exit_cleanup(scan_file):
    scan_file.close()


def wrapper(func, *args, **kwargs):
    def wrapped():
        return func(*args, **kwargs)
    return wrapped


def autocuratorf(autoc_command, fullmap, scanfn):
    os.system("bash gen-five/src/python/autocurator.sh " + autoc_command + " " + fullmap + " " + scanfn)


def main(fullmap):

    split_map = fullmap.split("/")
    fname = split_map[-1]
    fname_split = fname.split(".")
    proj = fname_split[0]
    cmip6 = False
    if proj == "CMIP6":
        cmip6 = True


    files = []
    files.append(fullmap)
    pub = args.get_args()
    third_arg_mkd = False
    if pub.json is not None:
        json_file = pub.json
        third_arg_mkd = True
        files.append(json_file)

    if cmip6 and pub.cmor_path == "":
        print("Error: No CMOR tables for PrePARE. Required for CMIP6.")
        exit(1)
    check_files(files)

    if pub.set_replica and pub.no_replica:
        print("ERROR: Replica simultaneously set and disabled.")
        exit(1)
    cert = pub.cert

    scan_file = tempfile.NamedTemporaryFile()  # create a temporary file which is deleted afterward for autocurator
    scanfn = scan_file.name  # name to refer to tmp file

    # add these as command line args
    cmor_tables = pub.cmor_path
    autocurator = pub.autocurator_path

    autoc_command = autocurator + "/bin/autocurator"  # concatenate autocurator command

    os.system("cert_path=" + cert)

    print("Converting mapfile...")
    try:
        map_json_data = mp.main([fullmap, proj])
    except Exception as ex:
        print("Error with converting mapfile: " + str(ex))
        exit_cleanup(scan_file)
        exit(1)
    print("Done.")

    if cmip6:
        try:
            prepare_internal(map_json_data, cmor_tables)
        except Exception as ex:
            print("Error with PrePARE: " + str(ex))
            exit_cleanup(scan_file)
            exit(1)

    # Run autocurator and all python scripts
    print("Running autocurator...")
    os.system("bash gen-five/src/python/autocurator.sh " + autoc_command + " " + fullmap + " " + scanfn)

    print("Done.\nMaking dataset...")
    try:
        if third_arg_mkd:
            out_json_data = mkd.main([map_json_data, scanfn, json_file])
        else:
            out_json_data = mkd.main([map_json_data, scanfn])

    except Exception as ex:
        print("Error making dataset: " + str(ex))
        exit_cleanup(scan_file)
        exit(1)

    if cmip6:
        print("Done.\nRunning pid cite...")
        try:
            new_json_data = pid.main(out_json_data)
        except Exception as ex:
            print("Error running pid cite: " + str(ex))
            exit_cleanup(scan_file)
            exit(1)

    print("Done.\nUpdating...")
    try:
        up.main(new_json_data)
    except Exception as ex:
        print("Error updating: " + str(ex))
        exit_cleanup(scan_file)
        exit(1)

    print("Done.\nRunning pub test...")
    try:
        pt.main(new_json_data)
    except Exception as ex:
        print("Error running pub test: " + str(ex))
        exit_cleanup(scan_file)
        exit(1)

    print("Done. Cleaning up.")
    exit_cleanup(scan_file)


if __name__ == '__main__':

    # os.system("export LD_LIBRARY_PATH=$CONDA_PREFIX/lib")  # this isn't working for some reason ...
    pub = args.get_args()
    fullmap = pub.map  # full mapfile path
    # allow handling of multiple mapfiles later
    if fullmap[-4:] != ".map":
        myfile = open(fullmap)
        for line in myfile:
            length = len(line)
            main(line[0:length-2])
        myfile.close()
        # iterate through file in directory calling main
    else:
        main(fullmap)



from esgcet.pid_cite_pub import ESGPubPidCite
from esgcet.activity_check import FieldCheck
import tempfile
from esgcet.generic_netcdf import GenericPublisher
import os
import esgcet.logger as logger

log = logger.ESGPubLogger()


class cmip6(GenericPublisher):

    scan_file = tempfile.NamedTemporaryFile()  # create a temporary file which is deleted afterward for autocurator
    scanfn = scan_file.name
    files = [scan_file, ]

    def __init__(self, argdict):
        super().__init__(argdict)
        self.pid_creds = argdict["pid_creds"]
        if "cmor_tables" in argdict:
            self.cmor_tables = os.path.expanduser(argdict["cmor_tables"])
        else:
            self.cmor_tables = None
        self.test = argdict["test"]
        self.publog = log.return_logger('CMIP6', self.silent, self.verbose)
        self._disable_citation = argdict["disable_citation"]


    def pid(self, out_json_data):
      
        pid = ESGPubPidCite(out_json_data, self.pid_creds, self.data_node, test=self.test,
                            silent=self.silent, verbose=self.verbose,
                            project_family='CMIP6', disable_cite=self._disable_citation)
        if self.cmor_tables:
            check = FieldCheck(self.cmor_tables, silent=self.silent)
            try:
                check.run_check(out_json_data)
            except:
                self.publog.exception("Activity/Metadata agreement check failed!")
                self.cleanup()
                exit(1)
            
        try:
            new_json_data = pid.do_pidcite(False)
        except Exception as ex:
            self.publog.exception("Assigning pid failed")
            self.cleanup()
            exit(1)

        return new_json_data

    def workflow(self):

        # step one: convert mapfile
        self.publog.info("Converting mapfile...")
        map_json_data = self.mapfile()

        # step two: extract
        self.publog.info(f"Running Extraction... {str(self.extract_method)}")
        self.extract_method(map_json_data)

        # step three: make dataset
        self.publog.info("Making dataset...")
        out_json_data = self.mk_dataset(map_json_data)

        # step four: assign PID
        self.publog.info("Assigning PID...")
        new_json_data = self.pid(out_json_data)
        
        #step five: update record if exists
        self.publog.info("Updating...")
        self.update(new_json_data)

        # step six: publish to database
        self.publog.info("Running index pub...")
        rc = self.index_pub(new_json_data)

        self.publog.info("Done. Cleaning up.")
        self.cleanup()
        return rc



import sys, json
from esgcet.mapfile import ESGPubMapConv
import configparser as cfg

from datetime import datetime, timedelta

from esgcet.settings import *
from pathlib import Path
from esgcet.mk_dataset import ESGPubMakeDataset
import esgcet.logger as logger

log = logger.ESGPubLogger()


class ESGPubMKDCreateIP(ESGPubMakeDataset):

    def init_project(self, s, l):

        self.project = "CREATE-IP"
        if s in self.source_ids:
            self.DRS = DRS["create-ip-src"]
        elif l == 7:
            self.DRS = DRS["create-ip-model"]
        elif l == 8:
            self.DRS = DRS["create-ip-var"]
        else:
            self.DRS = DRS["create-ip-exp"]

    def __init__(self, *args, **kwargs):
        super(ESGPubMakeDataset,self).__init__(args, kwargs)
        self.models = ["CCSM-CAM", "CFSR", "CREATE-MRE2models", "CREATE-MRE3models", "CREATE-MREmodels", "GEOS-5",
                   "IFS-Cy31r2", "IFS-Cy41r2", "JRA-25", "JRA-55", "MITgcm", "MOM3", "MOM4", "MRICOMv3",
                   "NCEP-Global-Operational-Model", "NEMOv3", "NEMOv32-LIM2", "NEMOv34-LIM2", "ORAmodels", "ensda-v351"]
        self.source_ids = [ "20CRv2c", "C-GLORSv5", "CERA-20C", "CFSR", "CREATE-MRE", "ECDAv31", "ERA-Interim", "ERA40-CRUTS3-10", "ERA5", "ERAInterim-CRUTS3-10", "GECCO2", "GODAS", "JRA-25", "JRA-55," "JRA-55-mdl-iso", "MERRA", "MERRA-CRUTS3-10", "MERRA2", "MERRA2-ASM", "MOVE-G2i", "MRE2ensemble", "NCEP-NCAR-CRUTS3-10", "ORAP5", "ORAS4", "ORAensemble"]
        self.publog = log.return_logger('Make Dataset CREATE-IP', self.silent, self.verbose)

    def get_dataset(self, mapdata, scanobj):

        master_id, version = mapdata.split('#')

        parts = master_id.split('.')
        exp_src = parts[3]

        projkey = parts[0]
        scandata = scanobj['dataset']

        if self.project:
            projkey = self.project
        self.init_project(exp_src, len(parts))

        facets = self.DRS  # depends on Init_project to initialize

        if not facets:
            raise RuntimeError()
        for var in list(scanobj["variables"].keys()):
            if "bnds" not in var and "_" not in var and "lon" not in var and "lat" not in var:
                self.variable = var

        for i, f in enumerate(facets):
            if f in scandata:
                ga_val = scandata[f]
                if not parts[i] == ga_val:
                    if f == "source_id" and ga_val not in self.source_ids:
                        self.dataset["experiment"] = ga_val
                    elif f == "experiment" and ga_val in self.source_ids:
                        self.dataset["source_id"] = ga_val
                    self.publog.warning("{} does not agree!\n".format(f) + ga_val)
            self.dataset[f] = parts[i]
        self.dataset[self.variable_name] = self.variable

        self.global_attributes(projkey, scandata)
        self.global_attr_mapped(projkey, scandata)
        self.const_attr()
        self.assign_dset_values(projkey, master_id, version)

    def aggregate_datasets(self, datasets, limit=False):
        vids = []
        v_long_names = []
        cf_std_names = []
        v_units = []
        last_dset = None
        last_rec = None
        for data in datasets:
            if data[0]["type"] == "Dataset":
                idx = 0
            elif data[-1]["type"] == "Dataset":
                idx = -1
            else:
                self.publog.error("No dataset record found. Exiting")
                exit(-4)
            dataset = data[idx]
            if self.variable_name in dataset and dataset[self.variable_name] not in vids:
                vids.append(dataset[self.variable_name])
            if "variable_long_name" in dataset and dataset["variable_long_name"] not in v_long_names:
                v_long_names.append(dataset["variable_long_name"])
            if "cf_standard_name" in dataset and dataset["cf_standard_name"] not in cf_std_names:
                cf_std_names.append(dataset["cf_standard_name"])
            if "variable_units" in dataset and dataset["variable_units"] not in v_units:
                v_units.append(dataset["variable_units"])
            last_rec = data
            last_dset = dataset
        if limit:
            last_dset[self.variable_name] = "Multiple"
            last_dset["variable_long_name"] = "Multiple"
            last_dset["cf_standard_name"] = "Multiple"
            last_dset["variable_units"] = "Multiple"
        else:
            last_dset[self.variable_name] = vids
            last_dset["variable_long_name"] = v_long_names
            last_dset["cf_standard_name"] = cf_std_names
            last_dset["variable_units"] = v_units
        last_rec[idx] = last_dset
        self.publog.debug("Aggregate record:\n" + json.dumps(last_dset, indent=4))
        return last_rec



from esgcet.pid_cite_pub import ESGPubPidCite
import argparse
import sys
import json
from pathlib import Path
import os
import esgcet.args as pub_args
import esgcet.logger as logger

log = logger.ESGPubLogger()
publog = log.return_logger('esgpidcitepub')


def get_args():
    parser = argparse.ArgumentParser(description="Publish data sets to ESGF databases.")
    home = str(Path.home())
    def_config = home + "/.esg/esg.yaml"
    parser.add_argument("--data-node", dest="data_node", default=None, help="Specify data node.")
    parser.add_argument("--pub-rec", dest="json_data", required=True,
                        help="Dataset and file json data; output from esgmkpubrec.")
    parser.add_argument("--config", "-cfg", dest="cfg", default=def_config, help="Path to yaml config file.")
    parser.add_argument("--out-file", dest="out_file", default=None,
                        help="Optional output file destination. Default is stdout.")
    parser.add_argument("--silent", dest="silent", action="store_true", help="Enable silent mode.")
    parser.add_argument("--verbose", dest="verbose", action="store_true", help="Enable verbose mode.")
    parser.add_argument("--test", dest="test", action="store_true",
                        help="PID registration will run in 'test' mode. Use this mode unless you are performing 'production' publications.")

    pub = parser.parse_args()

    return pub


def run():
    a = get_args()
    ini_file = a.cfg
    if not os.path.exists(ini_file):
        publog.error("Config file not found. " + ini_file + " does not exist.")
        exit(1)
    if os.path.isdir(ini_file):
        publog.error("Config file path is a directory. Please use a complete file path.")
        exit(1)
    args = pub_args.PublisherArgs()
    config = args.load_config(ini_file)

    p = True
    if a.out_file is not None:
        p = False
        outfile = a.out_file

    if a.data_node is None:
        try:
            data_node = config['data_node']
        except:
            publog.exception("Data node not supplied in config or command line. Exiting.")
            exit(1)

    if not a.silent:
        try:
            s = config['silent']
            if 'true' in s or 'yes' in s:
                silent = True
            else:
                silent = False
        except:
            silent = False
    else:
        silent = True

    if not a.verbose:
        try:
            v = config['verbose']
            if 'true' in v or 'yes' in v:
                verbose = True
            else:
                verbose = False
        except:
            verbose = False
    else:
        verbose = True

    if a.data_node is None:
        try:
            data_node = config['data_node']
        except:
            publog.exception("Data node not supplied in config or command line. Exiting.")
            exit(1)
    else:
        data_node = a.data_node

    test = False
    if a.test:
        test = True

    try:
        pid_creds = config['pid_creds']
    except:
        publog.exception("PID credentials not defined. Define in config file.")
        exit(1)

    try:
        out_json_data = json.load(open(a.json_data))
    except:
        publog.exception("Could not open JSON file. Exiting.")
        exit(1)

    pid = ESGPubPidCite(out_json_data, pid_creds, data_node, test=test, silent=silent,
                        verbose=verbose)

    try:
        new_json_data = pid.do_pidcite()
    except Exception as ex:
        publog.exception("Failed assigning pid or running activity check")
        exit(1)

    if p:
        print(json.dumps(new_json_data))
    else:
        with open(outfile, 'w') as of:
            json.dump(new_json_data, of)


def main():
    run()


if __name__ == '__main__':
    sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
    main()



from esgcet.args import PublisherArgs
import argparse
from pathlib import Path
import sys
import esgcet.args as pub_args
import json
import os
import esgcet.logger as logger

log = logger.ESGPubLogger()
publog = log.return_logger('esgmkpubrec')

from esgcet.mk_dataset_autoc import ESGPubAutocHandler
from esgcet.mk_dataset_xarray import ESGPubXArrayHandler

def get_args():
    parser = argparse.ArgumentParser(description="Publish data sets to ESGF databases.")

    # ANY FILE NAME INPUT: check first to make sure it exists
    home = str(Path.home())
    def_config = home + "/.esg/esg.yaml"
    parser.add_argument("--set-replica", dest="set_replica", action="store_true", help="Enable replica publication.")
    parser.add_argument("--no-replica", dest="no_replica", action="store_true", help="Disable replica publication.")
    parser.add_argument("--json", dest="json", default=None,
                        help="Load attributes from a JSON file in .json form. The attributes will override any found in the DRS structure or global attributes.")
    parser.add_argument("--scan-file", dest="scan_file", help="JSON output file from autocurator.  If not set will use XArray to load data from files")
    parser.add_argument("--map-data", dest="map_data", required=True,
                        help="Mapfile json data converted using esgmapconv.")
    parser.add_argument("--out-file", dest="out_file", default=None,
                        help="Optional output file destination. Default is stdout.")
    parser.add_argument("--data-node", dest="data_node", default=None, help="Specify data node.")
    parser.add_argument("--index-node", dest="index_node", default=None, help="Specify index node.")
    parser.add_argument("--project", dest="proj", default="",
                        help="Set/overide the project for the given mapfile, for use with selecting the DRS or specific features, e.g. PrePARE, PID.")
    parser.add_argument("--config", "-cfg", dest="cfg", default=def_config, help="Path to yaml config file.")
    parser.add_argument("--silent", dest="silent", action="store_true", help="Enable silent mode.")
    parser.add_argument("--verbose", dest="verbose", action="store_true", help="Enable verbose mode.")

    pub = parser.parse_args()

    return pub


def run():

    a = get_args()
    ini_file = a.cfg
    if not os.path.exists(ini_file):
        publog.error("Config file not found. " + ini_file + " does not exist.")
        exit(1)
    if os.path.isdir(ini_file):
        publog.error("Config file path is a directory. Please use a complete file path.")
        exit(1)
    args = pub_args.PublisherArgs()
    config = args.load_config(ini_file)

    p = True
    if a.out_file is not None:
        p = False
        outfile = a.out_file

    if not a.silent:
        try:
            s = config['silent']
            if 'true' in s or 'yes' in s:
                silent = True
            else:
                silent = False
        except:
            silent = False
    else:
        silent = True
        
    if not a.verbose:
        try:
            v = config['verbose']
            if 'true' in v or 'yes' in v:
                verbose = True
            else:
                    verbose = False
        except:
            verbose = False
    else:
        verbose = True

    try:
        map_json_data = json.load(open(a.map_data, 'r'))
    except:
        publog.exception("Missing required  Exiting.")
        exit(1)

    scanarg = a.scan_file
    
    if scanarg:
        format_handler = ESGPubAutocHandler
    else:
        format_handler = ESGPubXArrayHandler
        scanarg = format_handler.xarray_load(map_json_data)

    if a.data_node is None:
        try:
            data_node = config['data_node']
        except:
            publog.exception("Data node not supplied in config or command line. Exiting.")
            exit(1)
    else:
        data_node = a.data_node

    if a.index_node is None:
        try:
            index_node = config['index_node']
        except:
            publog.exception("Index node not supplied in config or command line. Exiting.")
            exit(1)
    else:
        index_node = a.index_node

    if a.set_replica and a.no_replica:
        publog.exception("Replica publication simultaneously set and disabled.")
        exit(1)
    elif a.set_replica:
        replica = True
    elif a.no_replica:
        replica = False
    else:
        try:
            r = config['set_replica']
            if 'yes' in r or 'true' in r:
                replica = True
            elif 'no' in r or 'false' in r:
                replica = False
            else:
                publog.error("Set_replica must be true, false, yes, or no.")
                exit(1)
        except:
            publog.exception("Set_replica not defined. Use --set-replica or --no-replica or define in config file.")
            exit(1)

    try:
        data_roots = config['data_roots']
        if data_roots == 'none':
            publog.error("Data roots undefined. Define in config file to create file metadata.")
            exit(1)
    except:
        publog.exception("Data roots undefined. Define in config file to create file metadata.")
        exit(1)

    try:
        globus = json.loads(config['globus_uuid'])
    except:
        # globus undefined
        globus = "none"

    try:
        dtn = config['data_transfer_node']
    except:
        # dtn undefined
        dtn = "none"

    json_file = None
    if a.json is not None:
        json_file = a.json

    if a.proj != "":
        project = a.proj
    else:
        try:
            project = config['project']
        except:
            project = None

    try:
        non_netcdf = config['non_netcdf'].lower()
        if 'yes' in non_netcdf or 'true' in non_netcdf:
            non_nc = True
        else:
            non_nc = False
    except:
        non_nc = False

    try:
        proj_config = json.loads(config['user_project_config'])
    except:
        proj_config = None


    construct = None

    if project == "create-ip":
        from esgcet.mkd_create_ip import ESGPubMKDCreateIP
        construct = ESGPubMKDCreateIP
    elif project == "cmip5":
        from esgcet.mkd_cmip5 import ESGPubMKDCmip5
        construct = ESGPubMKDCmip5
    elif project == "input4mips":
        from esgcet.mkd_input4mips import ESGPubMKDinput4MIPs
        construct = ESGPubMKDinput4MIPs
    elif non_nc:
        from esgcet.mkd_non_nc import ESGPubMKDNonNC
        construct = ESGPubMKDNonNC
    else:
        from esgcet.mk_dataset import ESGPubMakeDataset
        construct = ESGPubMakeDataset
    mkd = construct(data_node, index_node, replica, globus, data_roots, dtn, format_handler, silent, verbose)

    try:
        if non_nc:
            out_json_data = mkd.get_records(map_json_data, json_file)
        else:
            out_json_data = mkd.get_records(map_json_data, scanarg, json_file, user_project=proj_config)

    except Exception as ex:
        publog.exception("Failed to make dataset")
        exit(1)

    if p:
        print(json.dumps(out_json_data))
    else:
        with open(outfile, 'w') as of:
            json.dump(out_json_data, of)


def main():
    run()


if __name__ == '__main__':
    sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
    main()



from esgcet.mk_dataset import ESGPubMakeDataset
from esgcet.settings import GA, GA_DELIMITED
import esgcet.logger as logger

log = logger.ESGPubLogger()


class ESGPubMKDinput4MIPs(ESGPubMakeDataset):

    def __init__(self, data_node, index_node, replica, globus, data_roots, dtn, silent=False, verbose=False, limit_exceeded=False, user_project=None, skip_opendap=False):
        super().__init__(data_node, index_node, replica, globus, data_roots, dtn, silent, verbose, limit_exceeded, user_project, skip_opendap=skip_opendap)
        self.publog = log.return_logger('Make Dataset input4MIPs', silent, verbose)

    def xattr_handler(self):
        if not self.xattr:
            return {}
        if len(self.xattr) < 1:
            return {}
        return [x for x in self.xattr.values()][0]

    def global_attributes(self, proj, scandata):
        projkey = proj.lower()
        # handle Global attributes if defined for the project
        handler = self.xattr_handler()
        if projkey in GA:
            for facetkey in GA[projkey]:
                # did we find a GA in the data by the the key name

                if facetkey in handler:
                    facetval = handler[facetkey]
                elif facetkey in scandata:
                    facetval = scandata[facetkey]
                    # is this a delimited attribute ?
                self.dataset[facetkey] = facetval

    def proc_xattr(self, xattrfn):
        pass

    def get_records(self, mapdata, scanfilename, xattrfn=None, user_project=None):

        self.load_xattr(xattrfn)
        return super().get_records(mapdata, scanfilename)



import sys
from esgcet.mapfile import ESGPubMapConv
import json
import os
import argparse
from pathlib import Path
import esgcet.logger as logger
import esgcet.args as pub_args

log = logger.ESGPubLogger()
publog = log.return_logger('esgmapconv')


def get_args():
    parser = argparse.ArgumentParser(description="Publish data sets to ESGF databases.")

    home = str(Path.home())
    def_config = home + "/.esg/esg.yaml"
    parser.add_argument("--project", dest="proj", default="", help="Set/overide the project for the given mapfile, for use with selecting the DRS or specific features, e.g. PrePARE, PID.")
    parser.add_argument("--map", dest="map", required=True, help="Mapfile ending in .map extension, contains metadata about the record.")
    parser.add_argument("--out-file", dest="out_file", help="Output file for map data in JSON format. Default is printed to standard out.")
    parser.add_argument("--config", "-cfg", dest="cfg", default=def_config, help="Path to yaml config file.")

    pub = parser.parse_args()

    return pub


def run():
    a = get_args()
    ini_file = a.cfg
    if not os.path.exists(ini_file):
        publog.error("Config file not found. " + ini_file + " does not exist.")
        exit(1)
    if os.path.isdir(ini_file):
        publog.error("Config file path is a directory. Please use a complete file path.")
        exit(1)

    args = pub_args.PublisherArgs()
    config = args.load_config(ini_file)
    p = True
    if a.out_file is not None:
        p = False
        outfile = a.out_file

    proj = None
    if a.proj != "":
        proj = a.proj
    else:
        try:
            proj = config['project']
        except:
            pass

    try:
        fullmap = a.map
    except:
        publog.exception("Argparse error. Exiting.")
        exit(1)

    mapconv = ESGPubMapConv(fullmap)
    map_json_data = None
    try:
        map_json_data = mapconv.mapfilerun()

    except Exception as ex:
        publog.exception("Failed to convert mapfile")
        exit(1)

    if p:
        print(json.dumps(map_json_data))
    else:
        with open(outfile, 'w') as of:
            json.dump(map_json_data, of)


def main():
    run()


if __name__ == '__main__':
    sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
    main()



from ESGConfigParser import SectionParser
import configparser as cfg
import os, sys
from urllib.parse import urlparse
import shutil
from datetime import date
from pathlib import Path
import json
import esgcet.logger as logger
import yaml
import traceback

log = logger.ESGPubLogger()

DEFAULT_ESGINI = '/esg/config/esgcet/'
CONFIG_FN_DEST = "~/.esg/esg.yaml"

# These are the keys that 
MIGRATE_KEYS = ['pid_creds', 'data_roots', 'user_project_config']

def project_list(cfg_obj):
    return [x[0] for x in cfg_obj.get_options_from_table('project_options')]

class ESGPubMigrate(object):

    def __init__(self, i, newpath, silent=False, verbose=False):
        self.ini_path = i
        self.silent = silent
        self.verbose = verbose
        self.save_path = newpath
        self.publog = log.return_logger('esgmigrate', silent, verbose)

    def migrate_new(self):

        config = cfg.ConfigParser()
        ini_file = self.ini_path
        if os.path.isdir(ini_file):
            ini_file = os.path.join(ini_file, 'esg.ini')
        config.read(ini_file)        
        cf_user = config['user']

        cf_dict = {}
        for key in cf_user:
            if key in MIGRATE_KEYS:
                cf_val =  cf_user[key]
                if type(cf_val) is str and len(cf_val) > 1:
                    try:
                        print(cf_val)
                        cf_json = json.loads(cf_user[key])
                        cf_dict[key] = cf_json
                    except BaseException as ex:
                        self.publog.warn(f"Expected JSON string for {key} could not be parsed")
                        traceback.print_exc()
                else:
                    self.publog.warn(f"{key} expected in JSON format, has {cf_val}")
            else:
                cf_dict[key] = cf_user[key]
        del cf_dict['note']
        self.write_config(cf_dict)

    def project_migrate(self, project):

        if not project:
            return None
        path = self.ini_path
        SP = SectionParser("project:{}".format(project), directory=path)
        SP.parse(path)

        ret = {'DRS' : SP.get_facets('dataset_id')}
        try:
            ret['CONST_ATTR'] = { x[0] : x[1] for x in SP.get_options_from_table('category_defaults') }
        except:
            ret['CONST_ATTR'] = {}
            self.publog.debug("No category defaults found for {}".format(project))
        return ret


    def migrate(self, project=None):

        if not os.path.exists(os.path.join(self.ini_path, "esg.ini")):
            self.publog.error("Old config " + self.ini_path + "esg.ini not found or unreadable.")
            exit(1)

        try:
            sp = SectionParser('config:cmip6')
            sp.parse(self.ini_path)
        except Exception as e:
            self.publog.exception("Exception encountered.")
            return

        thredds_url = sp.get("thredds_url")
        res = urlparse(thredds_url)
        data_node = res.netloc


        index_node =''
        if 'rest_service_url' in sp:
            index_url = sp.get('rest_service_url')
            res = urlparse(index_url)
            index_node = res.netloc
        else:
            self.publog.warning("No REST publishing url found in old config, will need to set index_node setting")
        
        try:
            cmor_path = sp.get('cmor_table_path')
        except:
            cmor_path = "/usr/local/cmip6-cmor-tables/Tables"

        
        try:
            cmor_path = sp.get('cmor_table_path')
        except:
            cmor_path = "/usr/local/cmip6-cmor-tables/Tables"

        try:
            pid_creds_in = sp.get_options_from_table('pid_credentials')
        except:
            pid_creds_in = []

        pid_creds = []
        for i, pc in enumerate(pid_creds_in):
            rec = {}
            rec['url'] = pc[0]
            rec['port'] = pc[1]
            rec['vhost'] = pc[2]
            rec['user'] = pc[3]
            rec['password'] = pc[4]
            rec['ssl_enabled'] = bool(pc[5])
            rec['priority'] = i+1
            pid_creds.append(rec)

        try:
            data_roots = sp.get_options_from_table('thredds_dataset_roots')
        except:
            data_roots = []

        dr_dict = {}
        for dr in data_roots:
            dr_dict[dr[1]] = dr[0]

        try:
            svc_urls = sp.get_options_from_table('thredds_file_services')
        except:
            svc_urls = []

        DATA_TRANSFER_NODE = None
        GLOBUS_UUID = None

        for line in svc_urls:
            if line[0] == "GridFTP":
                res = urlparse(line[1])
                DATA_TRANSFER_NODE = res.netloc
            elif line[0] == "Globus":
                parts= line[1].split(':')
                GLOBUS_UUID = parts[1][0:36] # length of UUID

        cert_base = 'none'
        if 'heshessian_service_certfile' in sp:
            cert_base = sp.get('hessian_service_certfile')
        
        CERT_FN = cert_base.replace('%(home)s', '~')

        self.publog.debug(str(dr_dict))
        self.publog.debug(str(pid_creds))
        self.publog.debug(data_node)
        self.publog.debug(index_node)
        self.publog.debug(CERT_FN)
        self.publog.debug(DATA_TRANSFER_NODE)
        self.publog.debug(GLOBUS_UUID)
        self.publog.debug(project)

        project_config = {project: self.project_migrate(project)}
        new_config = {"data_node": data_node, "index_node": index_node, "data_roots": dr_dict, "cert": CERT_FN,
                      "globus_uuid": GLOBUS_UUID, "data_transfer_node": DATA_TRANSFER_NODE, "pid_creds": pid_creds,
                      "cmor_path" : cmor_path }
        if project_config:
            new_config["project_cfg"] = project_config
        self.write_config(new_config)

    def write_config(self, new_config):

        pid_creds = new_config['pid_creds']
        new_creds = {}
        for it in pid_creds:
            url = it['url']
            del it['url']
            new_creds[url] = it
        new_config['pid_creds'] = new_creds

        d = date.today()
        t = d.strftime("%y%m%d")
        config_file = self.save_path
        if os.path.exists(self.save_path):
            backup = f"{self.save_path}.{t}.bak"
            shutil.copyfile(config_file, backup)

        with open(config_file, 'w') as f:
            yaml.dump(new_config, f)



from esgcet.update_globus import ESGUpdateGlobus
import sys

UUID = "5cc79324-1b74-4a77-abc3-838aba2fc734"

def test(args):
    rec = [{},{"master_id" : args[0], 
               "data_node" : args[1]}]
    gup = ESGUpdateGlobus(UUID, sys.argv[1])

    gup.run(rec)
    
test(sys.argv[0:2])


import sys, json, os
from esgcet.index_pub import ESGPubIndex
import argparse
from pathlib import Path
import esgcet.logger as logger
import esgcet.args as pub_args

log = logger.ESGPubLogger()
publog = log.return_logger('esgindexpub')


def get_args():
    parser = argparse.ArgumentParser(description="Publish data sets to ESGF databases.")

    home = str(Path.home())
    def_config = home + "/.esg/esg.yaml"
    parser.add_argument("--index-node", dest="index_node", default=None, help="Specify index node.")
    parser.add_argument("--certificate", "-c", dest="cert", default="./cert.pem",
                        help="Use the following certificate file in .pem form for publishing (use a myproxy login to generate).")
    parser.add_argument("--pub-rec", dest="json_data", default=None,
                        help="JSON file output from esgpidcitepub or esgmkpubrec.")
    parser.add_argument("--config", "-cfg", dest="cfg", default=def_config, help="Path to yaml config file.")
    parser.add_argument("--silent", dest="silent", action="store_true", help="Enable silent mode.")
    parser.add_argument("--verbose", dest="verbose", action="store_true", help="Enable verbose mode.")
    parser.add_argument("--no-auth", dest="no_auth", action="store_true",
                        help="Run publisher without certificate, only works on certain index nodes.")
    parser.add_argument("--verify", dest="verify", action="store_true",
                        help="Toggle server certificate verification for publishing, default is off.")
    parser.add_argument("--xml-list", dest="xml_list", default=None,
                        help="Publish directly from xml files listed (supply a file containing paths to the files).")
    pub = parser.parse_args()

    return pub


def run():
    a = get_args()

    ini_file = a.cfg
    if not os.path.exists(ini_file):
        publog.error("Config file not found. " + ini_file + " does not exist.")
        exit(1)
    if os.path.isdir(ini_file):
        publog.error("Config file path is a directory. Please use a complete file path.")
        exit(1)
    args = pub_args.PublisherArgs()
    config = args.load_config(ini_file)

    if not (a.json_data or a.xml_list):
        publog.error("Input data argument missing.  Please provide either records in .json form or a list of xml files for index publishing")
        exit(1)

    if not (a.json_data or a.xml_list):
        publog.error("Input data argument missing.  Please provide either records in .json form or a list of xml files for index publishing")
        exit(1)

    if not a.silent:
        try:
            s = config['silent']
            if 'true' in s or 'yes' in s:
                silent = True
            else:
                silent = False
        except:
            silent = False
    else:
        silent = True

    if not a.verbose:
        try:
            v = config['verbose']
            if 'true' in v or 'yes' in v:
                verbose = True
            else:
                verbose = False
        except:
            verbose = False
    else:
        verbose = True

    if a.cert == "./cert.pem":
        try:
            cert = config['cert']
        except:
            cert = a.cert
    else:
        cert = a.cert

    if a.index_node is None:
        try:
            index_node = config['index_node']
        except:
            publog.exception("Index node not defined. Use the --index-node option or define in esg.ini.")
            exit(1)
    else:
        index_node = a.index_node

    if a.verify:
        verify = True
    else:
        verify = False

    if a.no_auth:
        auth = False
    else:
        auth = True

    rc = True
    ip = ESGPubIndex(index_node, cert, silent=silent, verbose=verbose, verify=verify, auth=auth)

    if a.json_data:
        try:
            new_json_data = json.load(open(a.json_data))
        except:
            publog.exception("Could not open json file. Exiting.")
            exit(1)
        try:
            rc = rc and ip.do_publish(new_json_data)
        except Exception as ex:
            publog.exception("Failed to publish to index node")
            exit(1)
    else:
        try:
            with open(a.xml_list) as inf:
                for line in inf:
                    ip.pub_xml(open(line.strip()).read())
        except Exception as ex:
            publog.exception(f"Index publishing failure: {ex}")
            exit(1)
    if not rc:
        exit(1)

def main():
    run()


if __name__ == '__main__':
    sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
    main()




from esgcet.esgmigrate import ESGPubMigrate
import argparse, sys, os
from pathlib import Path
import esgcet.logger as logger

log = logger.ESGPubLogger()
publog = log.return_logger('esgmigrate')


DEFAULT_ESGINI = '/esg/config/esgcet/'
home = str(Path.home())


def get_args():
    parser = argparse.ArgumentParser(description="Migrate old config settings into new format.")

    parser.add_argument("--old-config", dest="cfg", default=DEFAULT_ESGINI, help="Path to directory containing old config file to migrate.")
    parser.add_argument("--silent", dest="silent", action="store_true", help="Enable silent mode.")
    parser.add_argument("--verbose", dest="verbose", action="store_true", help="Enable verbose mode.")
    parser.add_argument("--project", dest="project", default=None, help='Name of a particular legacy project to migrate.')
    parser.add_argument("--destination", dest='dest', default=home+'/.esg/esg.ini', help="Destination for new config file.")
    parser.add_argument("--v5", dest="newcfg", action="store_true", help="Migrate a v5 esg.ini rather than a legacy version (v4 or earlier)")
    pub = parser.parse_args()

    return pub


def main():

    a = get_args()
    ini_path = a.cfg
    silent = a.silent
    verbose = a.verbose
    project = a.project
    filepath = a.dest

    try:
        em = ESGPubMigrate(ini_path, filepath, silent=silent, verbose=verbose)
        if a.newcfg:
            em.migrate_new()
        else:
            em.migrate(project)
    except:
        publog.exception("Failed to migrate old config")


if __name__ == "__main__":
    sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
    main()



import argparse
from pathlib import Path
import os
import esgcet.esgmigrate as em
import esgcet.logger as logger
import esgcet
import yaml

log = logger.ESGPubLogger()
publog = log.return_logger('Settings')

DEFAULT_ESGINI = '/esg/config/esgcet'


class PublisherArgs:
    """  Reconcile command line argumenets and config file settings.
    """
    def __init__(self):
        pass

    def get_args(self):
        """ Wrap argument parser
        """
        parser = argparse.ArgumentParser(description="Publish data sets to ESGF databases.")

        # ANY FILE NAME INPUT: check first to make sure it exists
        home = Path.home()
        def_config = home / ".esg/esg.yaml"
        parser.add_argument("--test", dest="test", action="store_true", help="PID registration will run in 'test' mode. Use this mode unless you are performing 'production' publications.")
        # replica stuff new... hard-coded, modify mk dataset so that it imports it instead
        parser.add_argument("--set-replica", dest="set_replica", action="store_true", help="Enable replica publication.")
        parser.add_argument("--no-replica", dest="no_replica", action="store_true", help="Disable replica publication.")
        parser.add_argument("--esgmigrate", dest="migrate", action="store_true", help="Run esgmigrate before publishing, to migrate over old config. May be left with incomplete config settings.")
        parser.add_argument("--json", dest="json", default=None, help="Load attributes from a JSON file in .json form. The attributes will override any found in the DRS structure or global attributes.")
        parser.add_argument("--data-node", dest="data_node", default=None, help="Specify data node.")
        parser.add_argument("--index-node", dest="index_node", default=None, help="Specify index node.")
        parser.add_argument("--certificate", "-c", dest="cert", default=None, help="Use the following certificate file in .pem form for publishing (use a myproxy login to generate).")
        parser.add_argument("--project", dest="proj", default="", help="Set/overide the project for the given mapfile, for use with selecting the DRS or specific features, e.g. PIDs.")
        parser.add_argument("--cmor-tables", dest="cmor_path", default=None, help="Path to CMIP CMOR tables for metadata checks. Required for CMIP6Plus and later.")
        parser.add_argument("--autocurator", dest="autocurator_path", default=None, help="Path to autocurator repository folder.")
        parser.add_argument("--map", dest="map", required=True, nargs="+", help="Mapfile or list of mapfiles.")
        parser.add_argument("--config", "-cfg", dest="cfg", default=str(def_config), help="Path to yaml config file.")
        parser.add_argument("--silent", dest="silent", action="store_true", help="Enable silent mode.")
        parser.add_argument("--verbose", dest="verbose", action="store_true", help="Enable verbose mode.")
        parser.add_argument("--no-auth", dest="no_auth", action="store_true", help="Run publisher without certificate, only works on certain index nodes.")
        parser.add_argument("--verify", dest="verify", action="store_true", help="Toggle verification for publishing, default is off.")
        parser.add_argument("--version", action="version", version=f"esgpublish v{esgcet.__version__}",help="Print the version and exit")
        parser.add_argument("--xarray", dest="xarray", action="store_true", help="Use Xarray to extract metadata even if Autocurator is configured.") 
        parser.add_argument("--no-xarray", dest="skipxr", action="store_true", help="Bypass use of Xarray (metadata will be incomplete)")
        pub = parser.parse_args()

        return pub

    def load_config(self, config_path):
        """
        Load the configuration file from specified path
        config_path : string  config file path cannot be empty
        """
        config_file = None
        try:
            config_file = open(config_path, 'r')  # or "a+", whatever you need
        except IOError:
            publog.error("Could not open file, please provide correct path to yaml config file.")
            quit(1)

        with config_file as fd:
            conf = yaml.load(fd, Loader=yaml.SafeLoader)
        return conf

    def get_dict(self,  fn_project):
        """
        Return a dict containing the publisher arguments to use:
        fn_project (string)  Specified project if pre-parsed.
        """
        pub = self.get_args()
        json_file = pub.json

        if pub.migrate:
            em.run(DEFAULT_ESGINI, False, False)

        config_file = pub.cfg
        config = self.load_config(config_file)
        if not os.path.exists(config_file):
            publog.error("Config file not found. " + config_file + " does not exist.")
            exit(1)
        if os.path.isdir(config_file):
            publog.error("Config file path is a directory. Please use a complete file path, exiting.")
            exit(1)

        if pub.proj != "":
            project = pub.proj
        else:
            try:
                project = config['project']
                if "none" in project:
                    project = fn_project
            except:
                project = fn_project

        verbose = False
        if not pub.silent:
            try:
                s = config['silent']
                if 'true' in s or 'yes' in s:
                    silent = True
                else:
                    silent = False
            except:
                silent = False
        else:
            silent = True

        if not pub.verbose:
            if not pub.silent:
                try:
                    v = config['verbose']
                    if 'true' in v or 'yes' in v:
                        verbose = True
                    else:
                        verbose = False
                except:
                    verbose = False
        else:
            verbose = True
            silent = False

        auth = False
        cert = ""
        if pub.cert:
            auth = True
            cert = pub.cert
        elif 'cert' in config:
            cert = config['cert']
            auth = True
        try:
            if pub.no_auth:
                auth = False
        except:
            pass

        if pub.xarray:
            autocurator = None
        elif pub.autocurator_path is None:
            autocurator = config.get('autoc_path', None)
            if autocurator == "none":
                autocurator = None
        else:
            autocurator = pub.autocurator_path

        if pub.index_node is None:
            try:
                index_node = config['index_node']
            except:
                publog.exception("Index node not defined. Use the --index-node option or define in esg.ini.")
                exit(1)
        else:
            index_node = pub.index_node

        if pub.data_node is None:
            try:
                data_node = config['data_node']
            except:
                publog.exception("Data node not defined. Use --data-node option or define in esg.ini.")
                exit(1)
        else:
            data_node = pub.data_node
        try:
            data_roots = config['data_roots']
            if data_roots == 'none':
                publog.error("Data roots undefined. Define in config file to create file metadata.")
                exit(1)
        except:
            publog.exception("Data roots undefined. Define in config file to create file metadata.")
            exit(1)

        try:
            globus = config['globus_uuid']
        except:
            globus = "none"


        disable_citation = config.get("disable_citation", False)
        disable_further_info = config.get("disable_further_info", False)
        
        if pub.set_replica and pub.no_replica:
            publog.error("Replica publication simultaneously set and disabled.")
            exit(1)
        elif pub.set_replica:
            replica = True
        elif pub.no_replica:
            replica = False
        else:
            replica = config.get('set_replica', False)
            
        test = False
        if pub.test:
            test = True

        try:
            proj_config = config['user_project_config']
        except:
            publog.warning("User project config missing or could not be parsed.")
            proj_config = {}


        if pub.verify:
            verify = True
        else:
            verify = False

        non_nc = config.get('non_netcdf', False)
      
        if not type(non_nc) is bool: 
            non_netcdf = config['non_netcdf'].lower()
            if 'yes' in non_netcdf or 'true' in non_netcdf:
                non_nc = True
            else:
                non_nc = False

        try:
            mountpoints = config['mountpoint_map']
            if mountpoints == "none":
                mountpoints = None
        except:
            mountpoints = None

        if globus == "none" and not silent:
            publog.info("No Globus UUID defined.")

        argdict = { "silent": silent, 
                   "verbose": verbose,
                   "autoc_command": autocurator, 
                   "index_node": index_node, 
                   "data_node": data_node,
                   "data_roots": data_roots, 
                   "globus": globus, 
                   "replica": replica,
                   "json_file": json_file, 
                   "test": test, 
                   "user_project_config": proj_config, 
                   "verify": verify,
                   "auth": auth, 
                   "non_nc": non_nc, 
                   "mountpoints": mountpoints,
                   "disable_citation": disable_citation,
                   "disable_further_info": disable_further_info}

        if auth and cert:
            argdict["cert"] = cert
            
        if project and "none" not in project:
            argdict["proj"] = project
            
        project = project.lower()
        if project == "cmip6":
            if pub.cmor_path is None:
                try:
                    argdict["cmor_tables"] = config['cmor_path']
                except:
                    publog.exception("No path for CMOR tables defined. Use --cmor-tables option or define in config file.")
                    exit(1)
            else:
                argdict["cmor_tables"] = pub.cmor_path
        if project == "cmip6" or project == "input4mips" or (project in proj_config and "pid_prefix" in proj_config[project]):  
            try:
                # Unpack the PID credentials format from the yaml to be compatible with the legacy format
                pid_creds = config['pid_creds']
                creds_lst = []
                for it in pid_creds:
                    rec = pid_creds[it]
                    rec['url'] = it
                    creds_lst.append(rec)
                argdict['pid_creds'] = creds_lst
            except:
                publog.exception("PID credentials not defined. Define in config file esg.ini.")
                exit(1)
        if "cmip6_clone" in config and project == config['cmip6_clone'].lower():
            if "pid_creds" in argdict:
                argdict["cmip6-clone"] = project
            if not argdict["user_project_config"]:
                argdict["user_project_config"] = {}
            argdict["user_project_config"]["clone_project"] = "cmip6"
        if "enable_archive" in config and config.get("enable_archive", False):
            try:
                argdict["enable_archive"] = True

                argdict["archive_path"] = config["archive_location"]
                argdict["archive_path_length"] = config["archive_depth"]
                if not os.path.isdir(argdict["archive_path"]):
                    publog.exception(f"Error with archive path {argdict['archive_path']}")
                    exit(1)
            except:
                publog.exception("Configuration file error: check archive (and other) settings")
                exit(1)
        else:
            argdict["enable_archive"] = False

        if "https_url" in config:
            argdict["https_url"] = config["https_url"]

        if config.get("globus_index", False):
            argdict["globus_index"] = True
            argdict["index_UUID"] = config.get("index_UUID", "")
        else:
            argdict["index_UUID"] =""
    
        argdict["dry_run"] = config.get("dry_run", False)
        
        if "skip_opendap" in config:
            argdict["skip_opendap"] = config["skip_opendap"]

        argdict["skipxr"] = pub.skipxr

        return argdict





import sys, json
from esgcet.mapfile import ESGPubMapConv
import configparser as cfg
import numpy as np

from datetime import datetime, timedelta

from esgcet.settings import *
from pathlib import Path
import esgcet.logger as logger

log = logger.ESGPubLogger()

class ESGPubMakeDataset:
    """
    Base class (abstract) to assemble the ESGF index records (dataset and file records).
    """
    def init_project(self, proj):
        """
        Intialize the specific project metadata based on a stock or custom configuration

        proj: Name of the project to be process
        """
        project = proj
        self.GA = GA
        
        if project in DRS:
            self.DRS = DRS[project]
            if project in CONST_ATTR:
                self.CONST_ATTR = CONST_ATTR[project]
        elif self.user_project and "clone_project" in self.user_project:
            cloneproj = self.user_project["clone_project"]
            if cloneproj not in DRS:
                raise(RuntimeError(f"Project {cloneproj} Data Record Syntax (DRS) not defined. Define in esg.ini"))
            self.DRS = DRS[cloneproj]
            if cloneproj in CONST_ATTR:
                self.CONST_ATTR = CONST_ATTR[cloneproj]
            else:
                self.CONST_ATTR = {}
            if 'CONST_ATTR' in self.user_project[project]:
                self.CONST_ATTR.update(self.user_project[project]['CONST_ATTR'])
        elif self.user_project and project in self.user_project:
            if 'DRS' in self.user_project[project]:
                self.DRS = self.user_project[project]['DRS']
            if 'CONST_ATTR' in self.user_project[project]:
                self.CONST_ATTR = self.user_project[project]['CONST_ATTR']
            if 'GA' in self.user_project[project]:
                self.GA = { project : self.user_project[project]['GA'] }
        else:
            raise (BaseException(f"Error: Project {project} Data Record Syntax (DRS) not defined. Define in esg.ini"))
        self.dataset['project'] = project
        
    def __init__(self, data_node, index_node, replica, globus, data_roots, https, handler_class=None, 
                 silent=False, verbose=False, limit_exceeded=False, user_project=None, disable_further_info=False, skip_opendap=False):
        """
        Constructor

        data_node (string):  ESGF Data Node to host the data 
        index_node (string):  ESGF Index Node to 
        replica (bool):  Is data a replca
        globus (string):  Globus endpoint UUID
        data_roots (dict): mapping of logical to file system roots for data
        https (string):  template for https urls if override, must be empty or None to use default from settings.py
        handler_class (class):  class type of the handler to construct the (format) handler object
        silent (bool):  Run in silent mode (suppress all output but errors)
        verbose (bool):  Print verbose (debug), 
        limit_exceeded (bool):
        user_project (dict):  User-defined project config info
        """
        self.silent = silent
        self.verbose = verbose
        self.data_roots = data_roots
        self.globus = globus
        self.data_node = data_node
        self.index_node = index_node
        self.replica = replica
        self._https_custom = https
        self.limit_exceeded = limit_exceeded

        self.mapconv = ESGPubMapConv("")
        self.dataset = {}
        self.project = None
        self.user_project = user_project
        self.DRS = None
        self.CONST_ATTR = None
        #  The default for variables, specific projects may use "variable" instead
        self.variable_name = "variable_id"
        self.publog = log.return_logger('Make Dataset', self.silent, self.verbose)
        self.xattr = None
        self.tracking_id_set = set()
        if handler_class:
            self.handler = handler_class(self.publog)
        self._disable_further_info = disable_further_info
        self.base_path = None #  This is used to create a directory for a dataset-level Globus url
        self._skip_opendap = skip_opendap

    def set_project(self, project_in):
        """
        Configure the project
        """
        self.project = project_in

    def prune_list(self, ll):
        """
        Shorten the list only actual items (no Nones)

        ll (list):  input list

        return list
        """
        for x in ll:
            if not x is None:
                yield (x)

    def load_xattr(self, xattrfn):
        """
        Load a set of "extended attributes" ie. additional key-value pairs for the dataset record from a file
        Default method, specific projects may format attributes diffeerntly
        xattrfn (string):  Full path to a .json file containing the pairs
        """
        if self.xattr:
            return
        if (xattrfn):
            self.xattr = json.load(open(xattrfn))
        else:
            self.xattr = {}

    def proc_xattr(self, xattrfn):
        """
        Load and process the extended attributes
        """
        self.load_xattr(xattrfn)
        if len(self.xattr) > 0:
            tmp_xattr = self.xattr_handler()
            for key in tmp_xattr:
                self.dataset[key] = tmp_xattr[key]

    def xattr_handler(self):
        """
        Base method for handling the extended attributes
        """
        return self.xattr

    def get_dataset(self, mapdata, scanobj):

        master_id, version = mapdata.split('#')
        parts = master_id.split('.')
        projkey = parts[0]
        self.first_val = projkey
        scandata = self.handler.get_attrs_dict(scanobj)

        if self.project:
            projkey = self.project
        self.init_project(projkey.lower())

        facets = self.DRS  # depends on Init_project to initialize

        if not facets:
            raise RuntimeError(f"Error DRS not configured for project {projkey}")
        if "variable" in facets:
            self.variable_name = "variable"

        for i, f in enumerate(facets):
            if f in scandata:
                ga_val = scandata[f]
                if not parts[i] == ga_val:
                    self.publog.warning("{} does not agree!".format(f))
            self.dataset[f] = parts[i]

        priorkey = projkey
        if self.user_project and "clone_project" in self.user_project:
            projkey = self.user_project["clone_project"]
        self.global_attributes(projkey, scandata)
        self.global_attr_mapped(projkey, scandata)
        self.assign_dset_values(master_id, version)
        self.const_attr()
        if not 'project' in self.dataset:
            self.dataset['project'] = priorkey
        if self._disable_further_info and "further_info_url" in self.dataset:
            self.publog.debug("Deleting further url field")
            del self.dataset["further_info_url"]
        

    def global_attributes(self, proj, scandata):
        # handle Global attributes if defined for the project
        projkey = proj.lower()

        if projkey in self.GA:
            for facetkey in self.GA[projkey]:
                # did we find a GA in the data by the the key name
                if facetkey in scandata:
                    facetval = scandata[facetkey]
                    # is this a delimited attribute ?
                    if projkey in GA_DELIMITED and facetkey in GA_DELIMITED[projkey]:
                        delimiter = GA_DELIMITED[projkey][facetkey]
                        self.dataset[facetkey] = facetval.split(delimiter)
                    else:
                        self.dataset[facetkey] = facetval

    def global_attr_mapped(self, proj, scandata):
        projkey = proj.lower()

        if projkey in GA_MAPPED:
            for gakey in GA_MAPPED[projkey]:
                if gakey in scandata:
                    facetkey = GA_MAPPED[projkey][gakey]
                    facetval = scandata[gakey]
                    self.dataset[facetkey] = facetval
                else:
                    self.publog.warning("GA to be mapped {} is missing!".format(facetkey))

    def const_attr(self):
        if self.CONST_ATTR:
            for facetkey in self.CONST_ATTR:
                self.dataset[facetkey] = self.CONST_ATTR[facetkey]

    def assign_dset_values(self, master_id, version):

        self.dataset['data_node'] = self.data_node
        self.dataset['index_node'] = self.index_node
        self.dataset['master_id'] = master_id
        self.dataset['instance_id'] = master_id + '.v' + version
        self.dataset['id'] = self.dataset['instance_id'] + '|' + self.dataset['data_node']
        if 'title' in self.dataset:
            self.dataset['short_description'] = self.dataset['title']
        self.dataset['title'] = self.dataset['master_id']
        self.dataset['replica'] = self.replica
        self.dataset['latest'] = True
        self.dataset['type'] = 'Dataset'
        self.dataset['version'] = int(version)


        fmat_list = ['%({})s'.format(x) for x in self.DRS]

        self.dataset['dataset_id_template_'] = '.'.join(fmat_list)
        self.dataset['directory_format_template_'] = '%(root)s/{}/%(version)s'.format('/'.join(fmat_list))


    def format_template(self, template, root, rel):
        if self._skip_opendap and "dodsC" in template:
            return None
        if "Globus" in template:
            if self.globus != 'none':
                return template.format(self.globus, root, rel)
            else:
                return None
        elif "HTTP" in template and self._https_custom:
            # Custom http template should have the hostname embedded
            return self._https_custom.format(root, rel)
        else:
            return template.format(self.data_node, root, rel)

    def gen_urls(self, proj_root, rel_path):
        res = self.prune_list([self.format_template(template, proj_root, rel_path) for template in URL_Templates])
        return list(res)

    def parse_path(self):
        assert self.base_path
        sp = self.base_path.split('/')
        res = '/'.join(sp[:-1])
        return res

    def get_file(self, mapdata, fn_trid):
        ret = self.dataset.copy()
        dataset_id = self.dataset["id"]
        ret['type'] = "File"
        fullfn = mapdata['file']

        fparts = fullfn.split('/')
        title = fparts[-1]
        ret['id'] = "{}.{}|{}".format(ret['instance_id'], title, self.data_node)
        ret['master_id'] = f"{ret['master_id']}.{title}"

        ret['instance_id'] = "{}.{}".format(ret['instance_id'], title)
        ret['title'] = title
        ret["dataset_id"] = dataset_id
        if "tracking_id" in fn_trid:
            if fn_trid["tracking_id"] in self.tracking_id_set:
                self.publog.error(f"Duplicate tracking_id {fn_trid['tracking_id']} encountered!")
                exit(1)
            self.tracking_id_set.add(fn_trid["tracking_id"])
            ret["tracking_id"] = fn_trid["tracking_id"]

        for kn in mapdata:
            if kn not in ("id", "file"):
                ret[kn] = mapdata[kn]

        rel_path, proj_root = self.normalize_path(fullfn, self.data_roots)
        if not proj_root in self.data_roots:

            # Need to handle the case where the root might contain the
            # project ID, ensure that the original root and the id are
            # found.`
            root_found = False
            for root in self.data_roots:
                if self.first_val in root and proj_root in root:
                    proj_root = root
                    rel_path = rel_path.replace(f"{self.first_val}/","")
                    root_found = True
                    if not self.base_path:
                        mapped_root = self.data_roots[root]
                        self.base_path = f"{mapped_root}/{rel_path}"

                    break

            if not root_found:
                self.publog.error('The file system root {} not found.  Please check your configuration.'.format(proj_root))
                exit(1)
        else:
            if not self.base_path:
                mapped_root = self.data_roots[proj_root]
                self.base_path = f"{mapped_root}/{rel_path}"
                        
        ret["url"] = self.gen_urls(self.data_roots[proj_root], rel_path)
        ret["publish_path"] = f"{self.data_roots[proj_root]}/{rel_path}" 
        if "number_of_files" in ret:
            ret.pop("number_of_files")
        else:
            self.publog.warning("No files present")
        if "datetime_start" in ret:
            ret.pop("datetime_start")
            ret.pop("datetime_end")

        return ret

    def set_variables(self, record, scanobj):
        variables = self.handler.get_variables(scanobj)
        # use the correct facet id string to get the variable if pre-specified in the record
        vid = record[self.variable_name]
        if vid in variables:
            var_rec = variables[vid]
            if "long_name" in var_rec:
                record["variable_long_name"] = var_rec["long_name"]
            elif "info" in var_rec:
                record["variable_long_name"] = var_rec["info"]
            if "standard_name" in var_rec:
                record["cf_standard_name"] = var_rec["standard_name"]
            if "units" in var_rec:
                record["variable_units"] = var_rec["units"]
            record[self.variable_name] = vid
            if self.variable_name == "variable_id":
                record["variable"] = vid
        else:
            var_list = self.handler.get_variable_list(variables)
            if len(var_list) < VARIABLE_LIMIT:
                init_lst = [self.variable_name, "variable_long_name"]
                if "variable_id" in init_lst:
                    init_lst.append("variable")
                for kid in init_lst:
                    record[kid] = []
                units_list = []
                cf_list = []
                for vk in var_list:
                    if not vk in VARIABLE_EXCLUDES:
                        var_rec = variables[vk]
                        if "long_name" in var_rec:
                            record["variable_long_name"].append(var_rec["long_name"])
                        elif "info" in var_rec:
                            record["variable_long_name"].append(var_rec["info"])
                        if "standard_name" in var_rec and len(var_rec["standard_name"]) > 0:
                            cf_list.append(var_rec["standard_name"]) 
                        if "units" in var_rec and var_rec["units"] != "1" and len(var_rec["units"]) > 0:
                            units_list.append(var_rec["units"])
                        record["variable"].append(vk)

                if self.variable_name == "variable_id":
                    record[self.variable_name] = "Multiple"
                record["variable_units"] = list(set(units_list))
                record["cf_standard_name"] = list(set(cf_list))
        
            if self.variable_name == "variable_id":
                record["variable"] = "Multiple"

    def update_metadata(self, record, scanobj):

        self.set_variables(record, scanobj)
        self.handler.set_bounds(record, scanobj)

    def iterate_files(self, mapdata, scandata):
        ret = []
        sz = 0
        last_file = None

        for maprec in mapdata:
            fullpath = maprec['file']
            if fullpath not in scandata.keys():
                if not self.limit_exceeded and self.project != "CREATE-IP" and self.project != "cmip5":
                    self.publog.error("Autocurator data not found for file: " + fullpath)
                    exit(1)
                continue
            scanrec = scandata[fullpath]
            file_rec = self.get_file(maprec, scanrec)
            last_file = file_rec
            sz += file_rec["size"]
            ret.append(file_rec)

        lst = []
        for x in last_file["url"]:
            if x:
                lst.append(x)
        last_file["url"] = lst
        access = [x.split("|")[2] for x in last_file["url"]]

        return ret, sz, access

    def get_records(self, mapdata, scanobj, xattrfn=None, user_project=None):

        self.user_project = user_project
        self.load_xattr(xattrfn)

        if isinstance(mapdata, str):
            mapobj = json.load(open(mapdata))
        else:
            mapobj = mapdata

        self.get_dataset(mapobj[0][0], scanobj)
        self.update_metadata(self.dataset, scanobj)
        self.dataset["number_of_files"] = len(mapobj)  # place this better
        project = self.dataset['project']

        self.publog.debug("Record:\n" + json.dumps(self.dataset, indent=4, default=lambda o: o.item() if isinstance(o, np.generic) else str(o)))

        self.mapconv.set_map_arr(mapobj)
        mapdict = self.mapconv.parse_map_arr()

        #self.publog.debug('Mapfile dictionary:\n' + json.dumps(mapdict, indent=1)) # Superverbose
       
        scandict = self.handler.get_scanfile_dict(scanobj, mapdict)
        #self.publog.debug('Autocurator Scanfile dictionary:\n' + json.dumps(scandict, indent=1))  # Superverbose

        ret, sz, access = self.iterate_files(mapdict, scandict)
        self.dataset["size"] = sz
        self.dataset["access"] = access

        self.dataset['globus_url'] = DATASET_GLOBUS_URL_TEMPLATE.format(self.globus, self.parse_path())
        self.proc_xattr(xattrfn)

        ret.append(self.dataset)
        return ret

    @staticmethod
    def normalize_path(path, data_roots):

        proj_root = None

        # Check each data root to see if it matches the provided path
        for data_root, _ in data_roots.items():
            path_match = "{}/".format(data_root.rstrip("/"))
            if path.startswith(path_match):

                rel_path = path[len(path_match):]
                proj_root = data_root.rstrip("/")

        if not proj_root:
            raise(BaseException("File Path does not match any data roots!"))

        return rel_path, proj_root



import netCDF4
from esgcet.handler_base import ESGPubHandlerBase
import os.path
import numpy as np

class ESGPubNC4Handler(ESGPubHandlerBase):

    @staticmethod
    def nc4_load(map_data):
        datafile = map_data[0][1]
        destpath = os.path.dirname(datafile)
        flst = os.listdir(destpath)

        if len(flst) == 0:
            raise RuntimeError("Directory empty!")
        fn = f"{destpath}/{flst[0]}"
        ds = netCDF4.Dataset(fn)

        return ds

    def get_attrs_dict(self, scanobj):
        return scanobj.__dict__

    def get_scanfile_dict(self, scandata, mapdata):
        ret = {}
        for rec in mapdata:
            fn = rec['file']
            ds = netCDF4.Dataset(fn)
            try:
                ret[fn] = {"tracking_id": ds.tracking_id}
            except:
                self.publog.warn("Tracking ID not found")
                ret[fn] = {}
        return ret
    
    def get_variables(self, scanobj):
        res = {}
        return res
    
    def get_variable_list(self, variable):
        return []

    
    def set_bounds(self, record, scanobj):

        pass




import xml.etree.ElementTree as ET
from xml.etree.ElementTree import ParseError
import re


def is_parseable_xml(s):
    try:
        tree = ET.fromstring(s)
        return True
    except ParseError:
        return False


def fixup_xml(s):
    """
    apply known tweaks to ensure parseable XML
    """
    if is_parseable_xml(s):
        return s

    # replace & with &amp; -- first try to preserve anything that looks like an HTML entity,
    # but if this doesn't work then just replace it unconditionally
    if "&" in s:
        s1 = re.sub("&(?![^\s]+;)", "&amp;", s)
        if is_parseable_xml(s1):
            s = s1
        else:
            s = s.replace('&', '&amp;')

    # add any more fixes here...
        
    if is_parseable_xml(s):
        return s
    else:
        raise Exception("Don't know a way to turn payload into parseable XML")
    
    
if __name__ == '__main__':
    for s in ('<doc><field name="foo">blah</field><field name="bar">blah &lt; blah</field></doc>',
              '<doc><field name="foo">bl & ah</field><field name="bar">blah &lt; blah</field></doc>',
              '<doc><field name="foo">bl & ah</field><field name="bar">blah &stuff; blah</field></doc>',
              '<doc><field name="foo">nohope'):
              print(s)
              print(is_parseable_xml(s))
              print(fixup_xml(s))
              print()



from abc import ABC, abstractmethod
import esgcet.logger as logger

log = logger.ESGPubLogger()


class ESGUpdateBase(ABC):


    def __init__(self, silent=False, verbose=False):
        self.publog = log.return_logger('Update Record', silent, verbose)

    @abstractmethod
    def update_file(self, dsetid : str):
        pass

    @abstractmethod
    def update_dataset(self, dsetid : str):
        pass

    @abstractmethod
    def query_update(self, data_node : str, master_id : str):
        pass

    def run(self, input_rec : dict):
        """ Check a record in the index and peform the updates

            input_rec - a json record to be published containing a "master_id" and "data_node" fields with 
                        a new version
        """
    # The dataset record either first or last in the input file
        dset_idx = -1
        if not input_rec[dset_idx]['type'] == 'Dataset':
            dset_idx = 0

        if not input_rec[dset_idx]['type'] == 'Dataset':
            self.publog.error("Could not find the Dataset record.  Malformed input, exiting!")
            exit(1)

        mst = input_rec[dset_idx]['master_id']
        dnode = input_rec[dset_idx]['data_node']

        dsetid = self.query_update(dnode, mst)

        if dsetid:
            self.update_dataset(dsetid)
            self.update_file(dsetid)
            self.publog.info('Found previous version, updating the record: {}'.format(dsetid))

        else:
            version = input_rec[dset_idx]['version']
            self.publog.info('First dataset version for {}: v{}.)'.format(mst, version))




from datetime import datetime

DRS = { 'CMIP6' : [ 'mip_era' , 'activity_drs','institution_id','source_id','experiment_id','member_id','table_id','variable_id','grid_label', 'version' ] }


from random import sample

def get_rand_lines(infile, count):

	return sample([line for line in infile], count)


def list_to_json(in_arr, node, **kwargs):

	increment = False
	if ('increment' in kwargs and kwargs['increment']):
		increment=True
	ret = []
	for line in in_arr:

		parts = line.split('.')

		key = parts[0]
		facets = DRS[key]
		d = {}
		for i, f in enumerate(facets):
			d[f] = parts[i]

		d['data_node'] = node
		d['index_node'] = node
		DRSlen = len(DRS[key])
		if increment:
			newvers = int(d['version'][1:]) + 1 
			d['version'] = newvers
			prev_id = '.'.join(parts[0:DRSlen])
			instance_id = '.'.join(parts[0:DRSlen - 1] + ['v' + str(newvers)])
			d['prev_id'] = prev_id
		else:
			instance_id = '.'.join(parts[0:DRSlen])
			d['version'] = d['version'][1:]
		d['instance_id'] = instance_id
		d['master_id'] = '.'.join(parts[0:DRSlen-1])
		d['id'] = instance_id + '|' + node
		d['title'] = instance_id
		d['replica'] = 'false'
		d['latest'] = 'true'
		d['type'] = 'Dataset'
		d['project'] = key + '-test'

		ret.append(d)

	return ret


def gen_xml(d):
    out = []
    out.append("<doc>\n")
    for key in d:

        val = d[key]
        if key == "description":
            val = ' '.join(val)
            out.append('  <field name="{}">{}</field>\n'.format(key, val))
        elif type(val) is list:
            for vv in val:
               out.append('  <field name="{}">{}</field>\n'.format(key, vv))
        else:
            out.append('  <field name="{}">{}</field>\n'.format(key, val))
    out.append("</doc>\n")
    return ''.join(out)

def write_xml(fn, txt, *args):
    pp = ""
    if len(args) > 0:
        pp = args[1]
    with open(pp + '/' + fn, 'w') as f:
        f.write(txt)


def gen_hide_xml(id, *args):

    dateFormat = "%Y-%m-%dT%H:%M:%SZ"
    now = datetime.utcnow()
    ts = now.strftime(dateFormat)
    txt =  """<updates core="datasets" action="set">
        <update>
          <query>instance_id={}</query>
          <field name="latest">
             <value>false</value>
          </field>
          <field name="_timestamp">
             <value>{}</value>
          </field>
        </update>
    </updates>
    \n""".format(id, ts)

    return txt

import sys

def main():
	d = list_to_json(get_rand_lines(sys.stdin, int(sys.argv[1])), 'esgf-test-data.llnl.gov', increment=True)

	path_prefix = ""

	if len(sys.argv) > 2:

		path_prefix = sys.argv[2]

	for rec in d:
		gen_hide_xml(rec['prev_id'], path_prefix)
		gen_xml(path_prefix+rec['instance_id'] + '.xml', rec)



if __name__ == '__main__':
	main()






import json, copy
import tempfile
import sys, os

from subprocess import Popen, PIPE
from time import sleep

FILTER_TEMPLATE = {
    "type" : "match_all",
    "field_name" : "data_node",
    "values" : []
    }

SEARCH_TEMPLATE = {
    "q" : "",
    "filters" : [
        ],
    }



class ESGGlobusQuery():

    def __init__(self, UUID_in, data_node_in):
        self._UUID = UUID_in
        self._data_node_filter = self._add_filter("data_node", data_node_in)

    def _add_filter(self, name, value, any=False):
        tmpfilter = copy.deepcopy(FILTER_TEMPLATE)
        tmpfilter["field_name"] = name
        if isinstance(value, list):
            tmpfilter["values"] += value
        else:
            tmpfilter["values"].append(value)
        if any:
            tmpfilter["type"] = "match_any"
        return tmpfilter
    
    def globus_get_record(self, subj):
        print(f"DEBUG {subj}")
        proc = Popen(["globus", "search", "subject", "show", self._UUID, subj[0].rstrip()], stdout=PIPE)
#        cmdstr = f"globus search subject show {self._UUID} '{subj.strip()}'"
#        proc = Popen(cmdstr            , shell=True, stdout=PIPE)
 #       print(f"DEBUG {cmdstr}")

 #       os.system(cmdstr)
        proc.wait()
#        res = proc.stdout.read().decode()
#        print(f"DEBUG {res}")
        return json.load(proc.stdout)

    def query_file_records(self, dataset_id, post_proc=True, latest=True, wget=False):
        q = copy.deepcopy(SEARCH_TEMPLATE)
        q["filters"].append(self._add_filter("type", "File"))
        q["filters"].append(self._add_filter("dataset_id", dataset_id, any=wget))
        if wget:
            latest = False

        if latest:
            q["filters"].append(self._add_filter("latest", True))

        print(f"Query in {q}")
        res = self._run_query(q, single=False, isjson=wget)
        
        if wget:
            return [entry["content"] for x in res["gmeta"] for entry in x["entries"]]
        elif post_proc:
            y = []
            for x in res:
                y.append(self._post_proc_query([x]))
            return y
        else:
            return res

    def dataset_query_master(self, master_id):
        self._dataset_query(master_id, "master_id")
    
    def _dataset_query(self, _id, field, latest=True):

        q = copy.deepcopy(SEARCH_TEMPLATE)
        q["filters"].append(self._data_node_filter)
        q["filters"].append(self._add_filter("type", "Dataset", any=True))
        q["filters"].append(self._add_filter(field, _id))
        if latest:
            q["filters"].append(self._add_filter("latest", True))

        res = self._run_query(q)

        return self._post_proc_query(res)

    def _run_query(self, q, single=False, isjson=False):
        temp = tempfile.NamedTemporaryFile(mode="w", delete=False)
        json.dump(q, temp, indent=1)
        print(f"DEBUG: to {temp.name}")
        temp.close()
    
        cmdarr = ["globus", "search", "query", self._UUID, "--query-document",temp.name, "--limit", "10000"]
        if isjson:
            cmdarr += ["--format", "json"]
        subproc = Popen(cmdarr, stdout=PIPE)

        sleep(5)
        
        #print("Complete")
        
        if isjson:
            res = {}
            try:
                res = json.load(subproc.stdout)
            except Exception as e:
                print(f"Exception encountered")
                with open("log.log", "w") as f:
                    print(f"{e}", file=f)
            print("Loaded")
            return res
        
        if single:
            subj = ""
            for x in subproc.stdout:
                if not subj:
                    raise RuntimeError(f"non empty {subj}")
                # There should be only one latest, TODO convert to raise an Exception
                subj = x.decode().rstrip()
        else:
            subj = []
            for x in subproc.stdout:
                subj.append(x.decode().rstrip())
  
        return subj

    def _post_proc_query(self, subj): 
        if subj:
            res = self.globus_get_record(subj)
            if res and "content" in res:
                return res["content"]
            else:
                raise RuntimeError(f"Unexpected error {res}")
        else:
            print("No record found")
            return {}

def test():        
    UUID = "5cc79324-1b74-4a77-abc3-838aba2fc734"
    DATA_NODE = "esgf-fake-test.llnl.gov"

    x = ESGGlobusQuery(UUID, DATA_NODE)

    res = x.dataset_query(sys.argv[1])
    with open("test.json", "w") as outf:
        json.dump(res, outf)
    

    res2 = x.file_query(res["id"])

    

#test()



from esgcet.args import PublisherArgs
import os
import sys
import esgcet.logger as logger

log = logger.ESGPubLogger()
publog = log.return_logger('Publisher-Main')

from pathlib import Path

def check_files(files):
    for file in files:
        try:
            myfile = open(file, 'r')
            myfile.close()
        except Exception as ex:
            publog.exception("Error opening file " + file + ". Exiting.")
            exit(1)


def run(fullmap, pub_args):

        # SETUP
    split_map = fullmap.split("/")
    fname = split_map[-1]
    fname_split = fname.split(".")
    project_name = fname_split[0]

    files = []
    files.append(fullmap)

    check_files(files)

    argdict = pub_args.get_dict(project_name)
    argdict["fullmap"] = fullmap

    if argdict["verbose"]:
        publog.info(argdict)
    if "proj" in argdict:
        project_name = argdict["proj"]
    else:
        argdict["proj"] = project_name
    project = project_name.lower()
    user_defined = False
    if argdict["user_project_config"]:
        user_defined = True
    non_netcdf = False
    if argdict["non_nc"]:
        non_netcdf = True

    if project == "cmip6" or "cmip6-clone" in argdict:
        from esgcet.cmip6 import cmip6
        proj = cmip6(argdict)
    elif project == "create-ip":
        from esgcet.create_ip import CreateIP
        proj = CreateIP(argdict)
    elif project == "cmip5":
        from esgcet.cmip5 import cmip5
        proj = cmip5(argdict)
    elif project == "input4mips":
        from esgcet.input4mips import input4mips
        proj = input4mips(argdict)
    elif project == "e3sm" and not non_netcdf:
        from esgcet.e3sm import e3sm
        proj = e3sm(argdict)
    elif non_netcdf:
        from esgcet.generic_pub import BasePublisher
        proj = BasePublisher(argdict)
    elif project == "generic" or project == "cordex" or user_defined or project == "none":
        if project == "none" and not argdict['silent']:
            publog.info("Using default settings, project not specified.")
        from esgcet.generic_netcdf import GenericPublisher
        proj = GenericPublisher(argdict)
    else:
        publog.error("Project " + project + " not supported.\nOpen an issue on our github to request additional project support.")
        exit(1)

    # ___________________________________________
    # WORKFLOW - one line call

    return proj.workflow()


def main():
    pub_args = PublisherArgs()
    pub = pub_args.get_args()
    maps = pub.map  # full mapfile path
    if maps is None:
        publog.error("Missing argument --map, use " + sys.argv[0] + " --help for usage.")
        exit(1)

    rc = True
    for m in maps:
        if os.path.isdir(m):
            mappath = Path(m)
            files = os.listdir(m)
            for f in files:
                fullmappath = mappath / f
                if os.path.isdir(fullmappath):
                    continue # Do not recurse subdirectories
                rc = rc and run(str(fullmappath), pub_args)
        else:
            myfile = open(m)
            ismap = False
            first = True
            for line in myfile:
                # if parsed line is not mapfile line, run on each file
                if first:
                    if '#' in line:
                        ismap = True
                        break
                    first = False

                length = len(line)
                rc = rc and run(line[0:length - 1], pub_args)
            myfile.close()
            if ismap:
                rc = rc and run(m, pub_args)

    if not rc:
        exit(1)


if __name__ == '__main__':
    sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
    main()



import sys, json, os
from esgcet.mapfile import ESGPubMapConv
import configparser as cfg
from esgcet.mk_dataset import ESGPubMakeDataset
from datetime import datetime, timedelta
from esgcet.settings import *
from pathlib import Path
import esgcet.logger as logger

log = logger.ESGPubLogger()


class ESGPubMKDNonNC(ESGPubMakeDataset):

    def __init__(self, data_node, index_node, replica, globus, data_roots,  silent=False, verbose=False, limit_exceeded=False, user_project=None):
        
        super().__init__(data_node, index_node, replica, globus, data_roots,  None, silent, verbose, limit_exceeded,
                         user_project)
        self.publog = log.return_logger('Make Non-NetCDF Dataset', silent, verbose)

    def get_dataset(self, mapdata):
        master_id, version = mapdata.split('#')

        parts = master_id.split('.')
        projkey = parts[0]
        if self.project:
            projkey = self.project
        self.init_project(projkey)

        facets = self.DRS

        for i, f in enumerate(facets):
            self.dataset[f] = parts[i]

        #    SPLIT_FACET = {'E3SM': {'delim': '_', 'facet': 'grid_resolution', 0: 'atmos_', 2: 'ocean_'}}
        if projkey in SPLIT_FACET:
            splitinfo = SPLIT_FACET[projkey]
            splitkey = splitinfo['facet']
            orgval = self.dataset[splitkey]
            valsplt = orgval.split(splitinfo['delim'])
            for idxkey in splitinfo:
                if type(idxkey) is int:
                    keyprefix = splitinfo[idxkey]
                    self.dataset[keyprefix + splitkey] = valsplt[idxkey]

        self.const_attr()
        self.assign_dset_values(master_id, version)
        if not 'project' in self.dataset:
            self.dataset['project'] = projkey
        
    def iterate_files(self, mapdata):
        ret = []
        sz = 0
        last_file = None

        for maprec in mapdata:
            fullpath = maprec['file']
            file_rec = self.get_file(maprec, {})
            last_file = file_rec
            sz += file_rec["size"]
            ret.append(file_rec)

        access = [x.split("|")[2] for x in last_file["url"]]

        return ret, sz, access

    def get_records(self, mapdata, xattrfn=None, user_project=None):
        self.user_project = user_project

        if isinstance(mapdata, str):
            mapobj = json.load(open(mapdata))
        else:
            mapobj = mapdata

        self.get_dataset(mapobj[0][0])

        self.dataset["number_of_files"] = len(mapobj)  # place this better
        project = self.dataset['project']

        self.proc_xattr(xattrfn)
        self.publog.debug("Record:\n" + json.dumps(self.dataset, indent=4))
        print()

        self.mapconv.set_map_arr(mapobj)
        mapdict = self.mapconv.parse_map_arr()
        self.publog.debug('Mapfile dictionary:\n' + json.dumps(mapdict, indent=4))
        print()

        ret, sz, access = self.iterate_files(mapdict)

        self.dataset["size"] = sz
        self.dataset["access"] = access
        ret.append(self.dataset)
        return ret



from esgcet.pub_client import publisherClient

import esgcet.list2json, sys, json
from esgcet.settings import INDEX_NODE, CERT_FN


def main(outdata):



    #	hostname = args[1]
    #	cert_fn = args[3]
    hostname = INDEX_NODE
    cert_fn = CERT_FN
    pubCli = publisherClient(cert_fn, hostname)

    d = outdata

    for rec in d:

        new_xml = esgcet.list2json.gen_xml(rec)
        print(new_xml)
        pubCli.publish(new_xml)
#        print(new_xml)



from esgcet.unpublish_globus import ESGUnpublishGlobus
from esgcet.unpublish_solr import ESGUnpublishSolr
import os
import sys
import json
import argparse
from pathlib import Path
import esgcet.args as pub_args
import esgcet.logger as logger

from esgcet.mapfile import ESGPubMapConv

log = logger.ESGPubLogger()
publog = log.return_logger('esgunpublish')

import esgcet

def get_args():
    parser = argparse.ArgumentParser(description="Unpublish data sets from ESGF databases.")

    home = str(Path.home())
    def_config = home + "/.esg/esg.yaml"
    parser.add_argument("--index-node", dest="index_node", default=None, help="Specify index node.")
    parser.add_argument("--data-node", dest="data_node", default=None, help="Specify data node.")
    parser.add_argument("--certificate", "-c", dest="cert", default=None,
                        help="Use the following certificate file in .pem form for unpublishing (use a myproxy login to generate).")
    parser.add_argument("--delete",  action="store_true", help="Specify deletion of dataset (default is retraction).")
    parser.add_argument("--deprecate",  action="store_true", help="Specify deprecation of dataset (default is retraction).")
    parser.add_argument("--dset-id", dest="dset_id", default=None,
                        help="Dataset ID for dataset to be retracted or deleted.")
    parser.add_argument("--map", dest="map", default=None, nargs="+",
                        help="Path(s) to a mapfile or directory(s) containing mapfiles.")    
    parser.add_argument("--use-list", dest="dset_list", default=None,
                        help="Path to a file containing list of dataset_ids.")
    parser.add_argument("--config", "-i", dest="cfg", default=def_config, help="Path to config file.")
    parser.add_argument("--version", action="version", version=f"esgunpublish v{esgcet.__version__}",help="Print the version and exit")
    parser.add_argument("--silent", dest="silent", action="store_true", help="Enable silent mode.")
    parser.add_argument("--verbose", dest="verbose", action="store_true", help="Enable verbose mode.")

    pub = parser.parse_args()

    return pub


def map_to_dataset(fullmap):

    mapconv = ESGPubMapConv(fullmap)
    map_json_data = None
    try:
        map_json_data = mapconv.mapfilerun()
        return map_json_data[0][0].replace("#",".v") 
    except Exception as ex:
        return None


def maps_to_dataset_list(maps):
    dset_list = []
    for m in maps:
        if os.path.isdir(m):
            os.listdir(m)
            if os.path.isdir(m):
                files = os.listdir(m)
                for f in files:
                    if os.path.isdir(m + f):
                        continue
                    dataset_id = map_to_dataset(m + f)
                    if not dataset_id is None:
                        dset_list.append(dataset_id)
        else:
            dataset_id = map_to_dataset(m)
            dset_list.append(dataset_id)

    return dset_list

def run():
    a = get_args()

    cfg_file = a.cfg
    if os.path.isdir(cfg_file):
        cfg_file = os.path.join(cfg_file, "esg.yaml")
    if not os.path.exists(cfg_file):
        publog.error(f"Config file not found. {cfg_file} does not exist.")
        raise RuntimeError(f"Config file not found. {cfg_file} does not exist.")

    args = pub_args.PublisherArgs()
    config = args.load_config(cfg_file)
    auth = False
    cert = ""

    if a.cert:
        cert = a.cert
    elif 'cert' in config:
        cert = config['cert']

    if cert:
        auth = True

    if a.index_node is None:
        try:
            index_node = config['index_node']
        except:
            publog.exception("Index node not defined. Use the --index-node option or define in esg.ini.")
            exit(1)
    else:
        index_node = a.index_node

    dset_id = ""
    if a.dset_id:
        dset_id = a.dset_id

    if not '|' in dset_id or (a.map):
        if a.data_node is None:
            try:
                data_node = config['data_node']
            except:
                publog.exception("Data node not defined. Use the --data-node option or define in esg.ini.")
                exit(1)
        else:
            data_node = a.data_node
    else:
        data_node = None

    if a.delete:
        d = True
    else:
        d = False

        
    if not a.silent:
        try:
            s = config['silent']
            if 'true' in s or 'yes' in s:
                silent = True
            else:
                silent = False
        except:
            silent = False
    else:
        silent = True

    if not a.verbose:
        if not a.silent:
            try:
                v = config['verbose']
                if 'true' in v or 'yes' in v:
                    verbose = True
                else:
                    verbose = False
            except:
                verbose = False
    else:
        verbose = True
        silent = False

    args = { "delete": d,
             "data_node": data_node,
             "index_node": index_node,
             "cert": cert,
             "auth" :auth,
             "verbose" : verbose,
             "silent" :silent,
             }

    if config.get("globus_index", False):
        args["index_UUID"] = config.get("index_UUID", "")
        upub = ESGUnpublishGlobus()
    else:
        upub = ESGUnpublishSolr()

    args["dry_run"] = config.get("dry_run", False)
    args["deprecate"] = a.deprecate

    if len(dset_id) > 0:
        args["dataset_id_lst"] = [dset_id]
    elif a.map:
        args["dataset_id_lst"] = maps_to_dataset_list(a.map)
    elif a.dset_list:
        try:
            dset_arr = [line.rstrip() for line in open(a.dset_list)]
        except:
            publog.exception(f"Error opening {args['dset_list']} file.")

        args["dataset_id_lst"] = dset_arr
    else:
        publog.error("No unpublish input method specified.  Please use from one of the following arguments: --map --use-list --dset-id ; type esgunpublish --help for more info")
        exit(1)

    if (upub.check_for_pid_proj(args["dataset_id_lst"])):
        try:
            pid_creds = config['pid_creds']
            creds_lst = []
            for it in pid_creds:
                rec = pid_creds[it]
                rec['url'] = it
                creds_lst.append(rec)
            args["pid_creds"] = creds_lst
        except:
            publog.exception("PID credentials not defined. Define in config file esg.ini.")
            exit(1)    

    status = 0
    try:
        status = upub.run(args)
    except Exception as ex:
        publog.exception("Failed to unpublish")
        exit(1)
    exit(status)

def main():
    run()


if __name__ == '__main__':
    sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
    main()



import json
import esgcet.logger as logger
from esgcet.settings import SOURCE_ID_LIMITS

log = logger.ESGPubLogger()


class FieldCheck(object):
    """
    Container for CV attribute agreement checks.
    For now this is specific to CMIP6.
    """
    def __init__(self, cmor_path, silent=False):
        """
        Constuctor.
        cmor_path (string)  Required path to CV files from a CMOR table repo
        silent (bool)  Run with out INFO messages
        """
        # TODO make this configurable per project
        cv_path = "{}/CMIP6_CV.json".format(cmor_path)
        jobj = json.load(open(cv_path))["CV"]
        self.sid_dict = jobj["source_id"]
        self.silent = silent
        self.idx = -1
        self.publog = log.return_logger('Activity Check', silent=silent)
        self.project_key = "cmip6"
        self.project_str = "CMIP6"

    def check_activity(self, source_id, activity_id):

        if source_id not in self.sid_dict:
            return False
        rec = self.sid_dict[source_id]

        return activity_id in rec["activity_participation"]

    def check_institution(self, source_id, inst_id):

        if source_id not in self.sid_dict:
            return False
        rec = self.sid_dict[source_id]

        return inst_id in rec["institution_id"]

    def run_check(self, input_rec):
        src_id = input_rec[self.idx]['source_id']
        act_id = input_rec[self.idx]['activity_drs']
        inst_id = input_rec[self.idx]['institution_id']

        if not src_id in self.sid_dict: 
            self.publog.error("Source_id {} is unregistered with the {} Controlled Vocabulary (CV). Publication halted".format(src_id, self.project_str))
            self.publog.error("If you think this message has been received in error, please update your CV source repository")
            raise UserWarning

        if self.project_key in SOURCE_ID_LIMITS and len(src_id) > SOURCE_ID_LIMITS[self.project_key]:
            self.publog.error(f"Source_id {src_id} exceeds the {SOURCE_ID_LIMITS[self.project_key]} character limit for project {self.project_str}. Publication halted.")
            raise UserWarning

        if not self.check_activity(src_id, act_id):
            self.publog.error("Source_id {} is not registered for participation in CMIP6 activity {}. Publication halted".format(src_id, act_id))
            self.publog.error("If you think this message has been received in error, please update your CV source repository")
            raise UserWarning
        if not self.check_institution(src_id, inst_id):
            self.publog.error("Institution_id {} is not registered to contribute to source_id {}. Publication halted".format(inst_id, src_id))
            self.publog.info("If you think this message has been received in error, please update your CV source repository")
            raise UserWarning

        self.publog.info("Passed source_id registration test for {}".format(src_id))
        



import logging


class ESGPubLogger:
    """
    Logger wrapper class
    """
    def __init__(self):
        """ Constructor
        """
        pass

    def return_logger(self, name, silent=False, verbose=False):
        """
        Logger 'factory' method allows for the naming and level specification.
        """
        publog = logging.getLogger(name)
        if silent:
            publog.setLevel(logging.WARNING)
        elif verbose:
            publog.setLevel(logging.DEBUG)
        else:
            publog.setLevel(logging.INFO)
        formatter = logging.Formatter(fmt='%(asctime)s %(levelname)-8s %(message)s', datefmt='%Y-%m-%d %H:%M:%S')
        handler = logging.StreamHandler()
        handler.setFormatter(formatter)
        publog.addHandler(handler)
        return publog



#!/usr/bin/env python

import os
import json
from datetime import datetime

from esgcet.settings import *

#  migrate to settings.py
NON_LIST = [
    "creation_date",
    "title",
    "data_node",
    "index_node",
    "master_id",
    "instance_id",
    "id",
    "replica",
    "latest",
    "type",
    "version",
    "north_degrees",
    "south_degrees",
    "east_degrees",
    "west_degrees",
    "datetime_start",
    "datetime_end",
    "number_of_files",
    "size",
    "timestamp",
    "deprecated",
    "dataset_id",
]

class GlobusSearchIngest:

    def __init__(self, doc_arr, cachedir=None) -> None:
        self._doc_arr = doc_arr
        self._cache_dir = cachedir

    def _get_gmeta_entry(self, doc, now=None):
        for key, value in doc.items():
            if isinstance(value, list):
                continue
            if key in NON_LIST:
                continue
            doc[key] = [value]

        doc["retracted"] = False
        if now:
            doc["_timestamp"] = now

        gmeta_entry = {
            "id": "dataset" if doc.get("type") == "Dataset" else "file",
            "subject": doc.get("id"),
            "visible_to": ["public"],
            "content": doc
        }

        return gmeta_entry
    def set_doc(self, doc):
        self._doc_arr = doc

    def convert2esgf2(self):
        pid_esgf1 = self._doc_arr
        gmeta = []
        # Don't replace an existing _timestamp should be immutable, add a mod_timestamp instead to capture the update
        now = None
        if not pid_esgf1[0].get("_timestamp", False):
            now = datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S.%fZ")

        # Transform an ESGF1 dataset entries to an ESGF2 Globus index entry
        for doc in pid_esgf1:
            gmeta_entry = self._get_gmeta_entry(doc, now)
            gmeta.append(gmeta_entry)

        # Create a GMetaList with a GMetaEntry for the dataset and GMetaEntries for files
        gingest = {
            "ingest_type": "GMetaList",
            "ingest_data": {
                "gmeta": gmeta
            }
        }

        return gingest

    def extern_globus_publish(self, filename, indexid):
        os.system(f"globus search ingest {indexid} {filename}")

    # pair with json.load(open(fn))
    def update_record(self, res, retract=False, deprecate=False):

        for suprec in res["ingest_data"]["gmeta"]:
            rec = suprec["content"]
            rec["latest"] = False
            ts_str = datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S.%fZ")
            rec["pub_timestamp"] = rec["_timestamp"]
            rec["_timestamp"] = ts_str
            if (retract):
                rec["retracted"] = True
                rec["dataset_status"] = "retracted"
            elif (deprecate):
                rec["deprecated"] = True
                rec["dataset_status"] = "deprecated"
        print(f"DEBUG {res}")
        return res

    def check_cache(self):

        tmp_filename, _ = self._get_cache_filename()
        if os.path.exists(tmp_filename):
            return json.load(open(tmp_filename))


    def _get_cache_filename(self):

        d2 = self._doc_arr 
        mid = d2[-1]['master_id']
        version= d2[-1]['version']
        parts = mid.split('.')    
        tmp_subpath =  '/'.join(parts[0:CACHE_DIR_DEPTH])

        if self._cache_dir:
            tmp_abspath = f'{self.cache_dir}/{tmp_subpath}'
        else:
            tmp_abspath = f'/tmp/.esg-publisher/{tmp_subpath}'
        tmp_filename = f'{tmp_abspath}/{mid}.v{version}.json'
        return tmp_filename, tmp_abspath
    
    def run(self, update=False, retract=False, deprecate=False):
        
        doc_res = self.convert2esgf2()
        if update:
            if deprecate:
                self.update_record(doc_res, retract=False, deprecate=True)
            else:
                self.update_record(doc_res, retract=retract)

        tmp_filename, tmp_abspath = self._get_cache_filename()
        print(f"makedirs {tmp_abspath}")
        os.makedirs(tmp_abspath, exist_ok = True)
        with open(tmp_filename, "w") as f2:
            print(json.dumps(doc_res), file=f2)
        return tmp_filename



import sys, json
from esgcet.settings import PID_PREFIX, PID_EXCHANGE, HTTP_SERVICE, CITATION_URLS, PID_URL
import traceback
import esgcet.logger as logger


log = logger.ESGPubLogger()

silent = False
verbose = False
pid_creds = {}
import traceback

class ESGPubPidCite(object):
    """ for PID services wraps calls to obtain a PID, add to records and generate citiation metadata """

    def __init__(self, ds_recs, pid_creds, data_node, test=False, silent=False, verbose=False, pid_prefix=PID_PREFIX,
                 project_family=None, disable_cite=False):
        """ Constructor
            ds_rec - a dataset record (dictionary/json)
            pid_creds - credentials typically loaded from config file, contains PID server, password, etc.
            silent, verbose - suppress info msgs, extend print
            test - register PID in test mode, essential for testing """
        self.ds_records = ds_recs
        self.pid_creds = pid_creds
        if type(self.pid_creds) == dict:
            self.pid_creds = [ self.pid_creds, ]
        self.silent = silent
        self.verbose = verbose
        self.test_publication = test
        self.pid_prefix = pid_prefix
        self.project_family = project_family
        self.data_node = data_node
        self.publog = log.return_logger('PID Citation', silent, verbose)
        self._disable_cite = disable_cite

    def establish_pid_connection(self):
        """Establish a connection to the PID service
        pid_prefix
            PID prefix to be used for given project
        test_publication
            Boolean to flag PIDs as test
        publish
            Flag to trigger publication and unpublication
        """
        try:
            import esgfpid
        except ImportError:
            self.publog.exception("PID module not found. Please install the package 'esgfpid' (e.g. with 'pip install').")
            exit(1)

        pid_messaging_service_exchange_name = PID_EXCHANGE
        pid_messaging_service_credentials = self.pid_creds
        pid_data_node = self.data_node

        # http_service_path = None

        # if publish:
        http_service_path = HTTP_SERVICE

        print(f" self.pid_connector = esgfpid.Connector(handle_prefix={self.pid_prefix}, \
                                          messaging_service_exchange_name={pid_messaging_service_exchange_name}, \
                                          messaging_service_credentials={pid_messaging_service_credentials}, \
                                          data_node={pid_data_node}, \
                                          thredds_service_path={http_service_path}, \
                                          test_publication={self.test_publication}")
        self.pid_connector = esgfpid.Connector(handle_prefix=self.pid_prefix,
                                          messaging_service_exchange_name=pid_messaging_service_exchange_name,
                                          messaging_service_credentials=pid_messaging_service_credentials,
                                          data_node=pid_data_node,
                                          thredds_service_path=http_service_path,
                                          test_publication=self.test_publication)
        # Check connection



    def check_pid_connection(self,  send_message=False):
        """
        Check the connection to the PID rabbit MQ
        Raise an Error if connection fails
        """
        pid_queue_return_msg = self.pid_connector.check_pid_queue_availability(send_message=send_message)
        if pid_queue_return_msg is not None:
            self.publog.error("Unable to establish connection to PID Messaging Service. Please check your esg.ini for correct pid_credentials.")



    def pid_flow_code(self):

        dsrec = {}
        try:
            dsrec = self.ds_records[-1]
        except:
            self.publog.exception("Failed to obtain dataset record: {}".format(str(self.ds_records)))
            exit(-1)
        if not 'data_node' in dsrec:
            self.publog.error("Record missing data node value")
            exit(-1)
        self.data_node = dsrec['data_node']
        dset = dsrec['master_id']
        version_number = dsrec['version']
        is_replica = dsrec["replica"]

        self.establish_pid_connection()
        self.pid_connector.start_messaging_thread()

        self.dataset_pid = None
        if self.pid_connector:
            self.dataset_pid = self.pid_connector.make_handle_from_drsid_and_versionnumber(drs_id=dset, version_number=version_number)
            self.publog.info("Assigned PID to dataset %s.v%s: %s " % (dset, version_number, self.dataset_pid))
        else:
            self.publog.warning('No connection')
        # if project uses citation, build citation url


        try:
            self.check_pid_connection(send_message=True)

            pid_wizard = None
                # Check connection
            pid_wizard = self.pid_connector.create_publication_assistant(drs_id=dset,
                                                                        version_number=version_number,
                                                                        is_replica=is_replica)
        # Iterate this over all the files:
            if len(self.ds_records) > 1:
                for file_rec in self.ds_records[0:-1]:

                    pid_wizard.add_file(file_name=file_rec['title'],
                                        file_handle=file_rec['tracking_id'],
                                        checksum=file_rec['checksum'],
                                        file_size=file_rec['size'],
                                        publish_path=file_rec['publish_path'],
                                        checksum_type=file_rec['checksum_type'],
                                        file_version=file_rec['version'] )
            else:
                file_rec = dsrec
                pid_wizard.add_file(file_name=file_rec['title'],
                                   file_handle=file_rec['tracking_id'],
                                   checksum=file_rec['checksum'],
                                   file_size=file_rec['size'],
                                   publish_path=file_rec['publish_path'],
                                   checksum_type=file_rec['checksum_type'],
                                   file_version=file_rec['version'])
            if pid_wizard:
                pid_wizard.dataset_publication_finished()
                return True
            else:
                self.publog.warning("Empty pid_wizard!")

        except Exception as e:
            traceback.print_exc()
            self.publog.exception("PID module exception encountered!")

        self.pid_connector.force_finish_messaging_thread()
        return False

    def pid_unpublish(self, drs_id, version):

        self.establish_pid_connection()
        self.pid_connector.start_messaging_thread()

        try:
            self.pid_connector.unpublish_one_version(drs_id=drs_id, version_number=version)
            self.pid_connector.finish_messaging_thread()
            return True
        except Exception as e:
            traceback.print_exc()
            self.pid_connector.force_finish_messaging_thread()
            self.publog.exception("PID module exception encountered!")

        self.pid_connector.force_finish_messaging_thread()
        return False


    def update_dataset(self, index):

        dset_rec = self.ds_records[index]
        # project is taken from the record metadata unless project_family is a truthy value
        # (defaults to None, but might be e.g. 'CMIP6')
        project = (self.project_family or dset_rec['project']).lower()
        # At present we only support the stock templates from CMIP6
        if not project in CITATION_URLS:
            return
        if self.test_publication:
            keystr = 'test'
        else:
            keystr = 'prod'

        
        dset_rec['pid'] = self.dataset_pid
        if not (dset_rec['type'] == 'File'):
            dset_rec['xlink'] = [PID_URL.format(self.dataset_pid)]

        if not self._disable_cite:

            citation_url = CITATION_URLS[project][keystr].format(dset_rec['master_id'], dset_rec['version'])

            dset_rec['citation_url'] = citation_url
            if not (dset_rec['type'] == 'File'):
                dset_rec['xlink'].append('{}|Citation|citation'.format(citation_url))

        self.ds_records[index] = dset_rec

    def do_pidcite(self):
        
        ret = self.pid_flow_code()

        if not ret:
            exit(-1)

        try:
            for i in range(len(self.ds_records)):
                self.update_dataset(i)

        except Exception as e:
            traceback.print_exc()
            self.publog.exception("Some exception encountered!")
            self.pid_connector.force_finish_messaging_thread()
            exit(-1)

        self.pid_connector.finish_messaging_thread()

        return self.ds_records

    def rewrite_json(self, fname):
        with open(fname, 'w') as f:
            f.write(json.dumps(self.ds_records, indent=1))


    def do_pidciterewrite(self, fname):
        res = self.do_pidcite()
        if type(res) is list:
            self.rewrite_json(fname)
        else:
            self.publog.warning("Something went wrong, PID/cite information were not added")




#    "xlink":["http://cera-www.dkrz.de/WDCC/meta/CMIP6/CMIP6.RFMIP.MOHC.HadGEM3-GC31-LL.rad-irf.r1i1p3f3.Efx.rld.gn.v20191030.json|Citation|citation",
 #         "http://hdl.handle.net/hdl:21.14100/2720a03c-479a-3cdf-99e2-1265d90d51ae|PID|pid"],,






import sys, json, requests
from datetime import datetime
from pathlib import Path
import esgcet.logger as logger

log = logger.ESGPubLogger()

''' Handles setting latest=false for previously published versions, includes finding those in the index
'''
class ESGSearchCheck():


    def __init__(self, index_node,  silent=False, verbose=False, verify=True):
        """
            index_node (string):  The node to search for the update 
            cert_fn (string):  Filename for certicate to use to push updates to the API
            silent (bool):  suppress INFO messages
            verbose (bool):  extended output, useful for debugging
        """
        self.index_node = index_node 
        self.silent = silent
        self.verbose = verbose
        self.verify = verify

        self.SEARCH_TEMPLATE = 'https://{}/esg-search/search/?format=application%2Fsolr%2Bjson&id={}&fields=id,retracted'
        self.publog = log.return_logger('Search Check', silent, verbose)



    def run_check(self, datasetid):
        """ Check a record in the index and peform the updates

            input_rec - a json record to be published containing a "master_id" and "data_node" fields with 
                        a new version
        """
    # The dataset record either first or last in the input file
        # query for
        url = self.SEARCH_TEMPLATE.format(self.index_node, datasetid)

        self.publog.debug("Search Url: '{}'".format(url))
        resp = requests.get(url, verify=self.verify)

        self.publog.debug(resp.text)
        if not resp.status_code == 200:
            self.publog.error('Received {} from index server.'.format(resp.status_code))
            exit(1)

        res = json.loads(resp.text)

        if res['response']['numFound'] > 0:
            docs = res['response']["docs"]
            if len(docs) < 1:
                raise RuntimeError("Error in response from index server, record not found!")
            dsetid = docs[0]['id']
            try:
                retracted = docs[0]['retracted']
            except KeyError: # support older publication without a retracted flag
                retracted = False
            if retracted:
                self.publog.info("Dataset already retracted")
                return True, False
            return True, True
        else:
            self.publog.info("Dataset not found")
            return False, False   



import sys, json
import os
from esgcet.pid_cite_pub import ESGPubPidCite
from esgcet.globus_query import ESGGlobusQuery
from esgcet.globus_search import GlobusSearchIngest


import esgcet.logger as logger

log = logger.ESGPubLogger()

import pdb

class ESGUnpublishGlobus:

    def __init__(self):
        self.publog = log.return_logger('esgunpublish-2')
    
    def check_for_pid_proj(self, dset_arr):
    
        # for dset in dset_arr:
    
        #     parts = dset.split('.')
        #     if parts[0].lower() in ["cmip6", "input4mips"]:
        #         return True            
        
        return False
    
    def run(self, args):

        self._args = args
        hostname = args["index_node"]
        data_node = args["data_node"]
        verbose = args["verbose"]    
        silent = args["silent"]
        auth = args["auth"]
        cert_fn = args["cert"]
        do_delete = args["delete"]
    
        pub_log = log.return_logger('Unpublish', args["silent"], args["verbose"])
        #searchcheck = ESGSearchCheck(hostname, silent, verbose)
        status = 0

        for dset_id in args["dataset_id_lst"]:
    
            status += self.single_unpublish(dset_id, pub_log)
        return status
    
    def single_unpublish(self, dset_id, pub_log):

        args = self._args
        
        hostname = args["index_node"]
        do_delete = args["delete"]
        data_node = args["data_node"]
        cert_fn = args["cert"]
        auth = args["auth"]
        self.UUID = args["index_UUID"]
        second_split = []
        if '|' in dset_id:
            first_split = dset_id.split('|')
            second_split = first_split[0].split('.')
            data_node = first_split[1]
        else:
            second_split = dset_id.split('.')
            dset_id_new = '{}|{}'.format(dset_id, data_node)
            dset_id = dset_id_new
    
    #    found, notretracted = searchcheck.run_check(dset_id)
    
        # if not found:
        #     return(-1)
        query= ESGGlobusQuery(self.UUID, data_node)
        try:
            res = query._post_proc_query([dset_id])
        except:
            return(-1)
            
        if args["deprecate"]:
            self.globus_deprecate(dset_id, query)
            res = query.query_file_records(dset_id, False)
            for x in res:
                self.globus_deprecate(x, query)  
            return (0)

        notretracted = not res["retracted"]
        
        if (not notretracted) and (not do_delete):
            pub_log.info("Use --delete to permanently erase the retracted record")
            return(0)
        
        if "pid_creds" in args and check_for_pid_proj([dset_id]):
            version = second_split[-1][1:]
            master_id = '.'.join(second_split[0:-1])
            pid_module = ESGPubPidCite({}, args["pid_creds"], data_node, False, args["silent"], args["verbose"])
            ret = pid_module.pid_unpublish(master_id, version)
            if not ret:
                pub_log.warning("PID Module did not return success")
        # ensure that dataset id is in correct format, use the set data node as a default
    
    
        
        # pubCli = publisherClient(cert_fn, hostname, auth=auth, verbose=args["verbose"], silent=args["silent"])
        self.data_node = data_node
        if do_delete:
            self.globus_delete_files(dset_id, query)
            self.globus_delete_subj(dset_id)
        else:
            self.globus_retract_files(dset_id, query)
            self.globus_retract_dataset(dset_id, query)
                
        #     pubCli.delete(dset_id)
        # else:
        #     pubCli.retract(dset_id)
        return(0)
    
    
    def globus_deprecate(self, dset_id, query):
        res = query._post_proc_query([dset_id])
        gs = GlobusSearchIngest([res])        

        res3 = gs.run(True, False, True)        
        if not self._args.get("dry_run", False):
            gs.extern_globus_publish(res3, self.UUID)
        else:
            self.publog.info(f"Dry Run publish-deprecate {res3}")

    def globus_retract_dataset(self, dset_id, query):
    
   #     query= ESGGlobusQuery(self.UUID, self.data_node)
        res = query._post_proc_query([dset_id])
        gs = GlobusSearchIngest([res])        

        res3 = gs.run(True, True)
        if not self._args.get("dry_run", False):
            gs.extern_globus_publish(res3, self.UUID)
        else:
            self.publog.info(f"Dry Run publish-retract {res3}")
    
    def globus_delete_files(self, dset_id, query):
        res = query.query_file_records(dset_id, False)
        for x in res:
            self.globus_delete_subj(x)
            
    def globus_retract_files(self, dset_id, query):
        res = query.query_file_records(dset_id, False)
        for x in res:
            self.globus_retract_dataset(x, query)    
        
    def globus_delete_subj(self,  subj):

        if not self._args.get("dry_run", False):
            os.system(f"globus search subject delete {self.UUID} '{subj}'") 
        else:
            self.publog.info(f"Dry Run delete {subj}")
    



from abc import ABC, abstractmethod


from esgcet.globus_query import ESGGlobusQuery
from esgcet.globus_search import GlobusSearchIngest
from esgcet.update_base import ESGUpdateBase

class ESGUpdateGlobus(ESGUpdateBase):

    def __init__(self, index_UUID : str, data_node : str, silent : bool = False, verbose : bool = False, dry_run : bool = False):
        ESGUpdateBase.__init__(self, silent=silent, verbose=verbose)
        self._index_UUID = index_UUID
        self._query = ESGGlobusQuery(self._index_UUID, data_node)
        self._dry_run = dry_run

    def update_file(self, dsetid : str):
        # Check cache
        #
        gs = GlobusSearchIngest({})
        #gs.check_cache()   

        filerecs = self._query.query_file_records(dsetid)
        gs.set_doc(filerecs)
        res = gs.run(True)
        if not self._dry_run:
            gs.extern_globus_publish(res, self._index_UUID)
        else:
            print(f"DEBUG dry run {res}")
        # publish file recs

    def update_dataset(self, dsetid : str):

        gs = GlobusSearchIngest([self._dataset_ctx])        
        res = gs.run(True)
        if not self._dry_run:
            gs.extern_globus_publish(res, self._index_UUID)
        else:
            print(f"DEBUG dry run {res}")

    def query_update(self, data_node, master_id : str):
        
   
        res = self._query.dataset_query_master(master_id)
        if (res):
            self._dataset_ctx = res
            print(f"DEBUG {res}")
            subject = res['id']  # todo get subject
            return subject
        else:
            return ""
        
        
        



from esgcet.pub_client import publisherClient
import sys, json, requests
from datetime import datetime
from pathlib import Path
import esgcet.logger as logger

log = logger.ESGPubLogger()

''' Handles setting latest=false for previously published versions, includes finding those in the index
'''
class ESGPubUpdate():


    def __init__(self, index_node, cert_fn="", silent=False, verbose=False, verify=False, auth=False):
        """
            index_node (string):  The node to search for the update 
            cert_fn (string):  Filename for certicate to use to push updates to the API
            silent (bool):  suppress INFO messages
            verbose (bool):  extended output, useful for debugging
        """
        self.index_node = index_node 
        self.cert_fn = cert_fn
        self.silent = silent
        self.verbose = verbose
        self.verify = verify
        self.pubCli = publisherClient(self.cert_fn, self.index_node, verify=verify, verbose=self.verbose, silent=self.silent, auth=auth)

        self.SEARCH_TEMPLATE = 'https://{}/esg-search/search/?latest=true&distrib=false&format=application%2Fsolr%2Bjson&data_node={}&master_id={}&fields=version,id'
        self.publog = log.return_logger('Update Record', silent, verbose)

    def gen_hide_xml(self, id, type):
        ''' Generate the xml to hide the previous version

            id - the full dataset identifier
            type - which core to target: "files" or datasets"
        '''

        dateFormat = "%Y-%m-%dT%H:%M:%SZ"
        now = datetime.utcnow()
        ts = now.strftime(dateFormat)
        idfield = "id"
        if type == "files":
            idfield = "dataset_id"
        txt = f"""<updates core="{type}" action="set">
            <update>
              <query>{idfield}={id}</query>
              <field name="latest">
                 <value>false</value>
              </field>
              <field name="_timestamp">
                 <value>{ts}</value>
              </field>
            </update>
            </updates>
            \n"""

        return txt

    def globus_update():

        # 
        pass         

    def update_core(self, dsetid, type):
        """ For a specific core, generate the xml and run the update.

            id - the full dataset identifier
            type - which core to target: "files" or datasets"
        """
        update_rec = self.gen_hide_xml(dsetid, type)
        self.publog.debug(update_rec)
        self.pubCli.update(update_rec)

    def run(self, input_rec):
        """ Check a record in the index and peform the updates

            input_rec - a json record to be published containing a "master_id" and "data_node" fields with 
                        a new version
        """
    # The dataset record either first or last in the input file
        dset_idx = -1
        if not input_rec[dset_idx]['type'] == 'Dataset':
            dset_idx = 0

        if not input_rec[dset_idx]['type'] == 'Dataset':
            self.publog.error("Could not find the Dataset record.  Malformed input, exiting!")
            exit(1)

        mst = input_rec[dset_idx]['master_id']
        dnode = input_rec[dset_idx]['data_node']

        dsetid = self.query_esg_search(dnode, mst)

        if dsetid:
            self.update_core(dsetid,"datasets")
            self.update_core(dsetid, "files")
            self.publog.info('Found previous version, updating the record: {}'.format(dsetid))

        else:
            version = input_rec[dset_idx]['version']
            self.publog.info('First dataset version for {}: v{}.)'.format(mst, version))


    def query_esg_search(self, dnode, mst):

        # query for
        url = self.SEARCH_TEMPLATE.format(self.index_node, dnode, mst)

        self.publog.debug("Search Url: '{}'".format(url))
        resp = requests.get(url, verify=self.verify)

        self.publog.debug(resp.text)
        if not resp.status_code == 200:
            self.publog.error('Received {} from index server.'.format(resp.status_code))
            exit(1)

        res = json.loads(resp.text)

        if res['response']['numFound'] > 0:
            docs = res['response']["docs"]
            dsetid = docs[0]['id']
            return dsetid
        else:
            return ""
    



import os
from esgcet.generic_netcdf import GenericPublisher
from esgcet.mkd_create_ip import ESGPubMKDCreateIP
from esgcet.update import ESGPubUpdate
from esgcet.index_pub import ESGPubIndex
import tempfile
from esgcet.settings import VARIABLE_LIMIT
from copy import deepcopy
import esgcet.logger as logger

log = logger.ESGPubLogger()


class CreateIP(GenericPublisher):

    def __init__(self, argdict):
        super().__init__(argdict)

        self.scans = []
        self.datasets = []
        self.variables = []
        self.master_dataset = None
        self.variable_limit = 75
        self.autoc_args = ' --out_pretty --out_json {} --files "{}/{}_*.nc"'
        self.publog = log.return_logger('CREATE-IP', self.silent, self.verbose)

    def cleanup(self):
        for scan in self.scans:
            scan.close()

    def autocurator(self, map_json_data):
        datafile = map_json_data[0][1]
        destpath = os.path.dirname(datafile)

        autstr = self.autoc_command + self.autoc_args
        files = os.listdir(destpath)
        for f in files:
            var = f.split('_')[0]
            if var not in self.variables:
                self.variables.append(var)
        for var in self.variables:
            self.scans.append(
                tempfile.NamedTemporaryFile())  # create a temporary file which is deleted afterward for autocurator
            scan = self.scans[-1].name
            self.publog.info("Autocurator command: " + autstr.format(scan, destpath, var))
            stat = os.system(autstr.format(scan, destpath, var))
            if os.WEXITSTATUS(stat) != 0:
                self.publog.error("Autocurator exited with exit code: " + str(os.WEXITSTATUS(stat)))
                self.cleanup()
                exit(os.WEXITSTATUS(stat))

    def mk_dataset(self, map_json_data):
        limit_exceeded = len(self.variables) > VARIABLE_LIMIT
        limit = False
        mkd = ESGPubMKDCreateIP(self.data_node, self.index_node, self.replica, self.globus, self.data_roots,
                                self.dtn, self.silent, self.verbose, limit_exceeded)
        for scan in self.scans:
            try: 
                out_json_data = mkd.get_records(map_json_data, scan.name, self.json_file)
                self.datasets.append(deepcopy(out_json_data)) # herein lies the issue, copy hasn't fixed it
            except Exception as ex:
                self.publog.exception(f"Failed to make dataset: {ex}")
                self.cleanup()
                exit(1)
            # only use first scan file if more than 75 variables
            if len(self.variables) > self.variable_limit:
                limit = True
                break

        self.master_dataset = mkd.aggregate_datasets(self.datasets, limit)
        return 0

    def update(self, placeholder):
        up = ESGPubUpdate(self.index_node, self.cert, silent=self.silent, verbose=self.verbose, verify=self.verify, auth=self.auth)
        try:
            up.run(self.master_dataset)
        except Exception as ex:
            self.publog.exception(f"Failed to update record: {ex}")
            self.cleanup()
            exit(1)

    def index_pub(self, placeholder):
        ip = ESGPubIndex(self.index_node, self.cert, silent=self.silent, verbose=self.verbose, verify=self.verify, auth=self.auth)
        try:
            ip.do_publish(self.master_dataset)
        except Exception as ex:
            self.publog.exception(f"Failed to publish to index node: {ex}")
            self.cleanup()
            exit(1)



import sys, json
from esgcet.mapfile import ESGPubMapConv
import configparser as cfg

from datetime import datetime, timedelta

from esgcet.settings import *
from pathlib import Path
from esgcet.mk_dataset import ESGPubMakeDataset
from esgcet.mkd_create_ip import ESGPubMKDCreateIP
import esgcet.logger as logger

log = logger.ESGPubLogger()


class ESGPubMKDCmip5(ESGPubMKDCreateIP):

    def init_project(self):

        self.DRS = DRS["cmip5"]
        self.project = "cmip5"

    def __init__(self, *args, **kwargs):
        super(ESGPubMKDCmip5, self).__init__(args, kwargs)

        self.variable_name = "variable"
        self.publog = log.return_logger('Make Dataset CMIP5', self.silent, self.verbose)

    def get_dataset(self, mapdata, scanobj):

        master_id, version = mapdata.split('#')

        parts = master_id.split('.')

        self.init_project()

        facets = self.DRS  # depends on Init_project to initialize

        if not facets:
            raise RuntimeError()
        for var in list(scanobj["variables"].keys()):
            if "bnds" not in var and "_" not in var and "lon" not in var and "lat" not in var:
                self.variable = var
        for i, f in enumerate(facets):
            self.dataset[f] = parts[i]
        self.dataset[self.variable_name] = self.variable

#        self.global_attributes(projkey, scandata)
#        self.global_attr_mapped(projkey, scandata)
        self.const_attr()
        self.assign_dset_values(self.project, master_id, version)




#!/usr/bin/env python
    
from setuptools import setup, find_packages
from pathlib import Path
import os
import sys
import configparser as cfg
from shutil import copyfile

import esgcet
VERSION = esgcet.__version__

additional_requirements = [
            "xarray",
            "netcdf4",
            "dask",
            "pyyaml",
            "globus-cli"
]


setup(
    name = 'esgcet',
    version = VERSION,
    description = 'ESGCET publication package',
    author = 'Sasha Ames',
    author_email = 'ames4@llnl.gov',
    url = 'http://esgf.github.io',
    install_requires = [
        "requests",
            "esgfpid",
        "ESGConfigParser==1.0.0a1",
    ] + additional_requirements,
    include_package_data = True,
    zip_safe = False,                   # Migration repository must be a directory
    entry_points={'console_scripts': ['esgpidcitepub=esgcet.esgpidcitepub:main',
                                        'esgmkpubrec=esgcet.esgmkpubrec:main',
                                        'esgindexpub=esgcet.esgindexpub:main',
                                        'esgpublish=esgcet.pub_internal:main',
                                        'esgupdate=esgcet.esgupdate:main',
                                        'esgmapconv=esgcet.esgmapconv:main',
                                        'esgmigrate=esgcet.migratecmd:main',
                                        'esgunpublish=esgcet.esgunpublish:main']}
)



import json
from datetime import datetime
import traceback
import esgcet.logger as logger

log = logger.ESGPubLogger()

class ESGPubMapConv:

    def __init__(self, mapfilename, project=None, silent=False):

        self.mapfilename = mapfilename
        self.project = project
        self.map_data_arr = []
        self.map_json = {}
        self.silent = silent
        self.publog = log.return_logger('Mapfile Conversion', silent=silent)

    def parse_map(self, mountpoints=None):
        """  """
        ret = []
        for line in self.map_data:

            parts = line.rstrip().split(' | ')
            if mountpoints and self.project:
                mapstr = parts[1]
                root = mapstr.split(self.project)[0][:-1]
                if root in mountpoints:
                    parts[1] = mapstr.replace(root, mountpoints[root])

            ret.append(parts)

        self.map_data_arr = ret
        return ret

    def set_map_arr(self, maparr):
        self.map_data_arr = maparr

    def parse_map_arr(self):
        ''' Input: Takes a 2-D array representation of the parsed map.
        Returns: file records.  assumes that the files all belong to the same dataset
        '''
        if len(self.map_data_arr) == 0:
            self.publog.warning("Empty map data")

        ret = []
        for lst in self.map_data_arr:
            rec = {}
            rec['file'] = lst[1]
            rec['size'] = int(lst[2])
            for x in lst[3:]:
                parts = x.split('=')
                if parts[0] == 'mod_time':
                    rec["timestamp"] = datetime.utcfromtimestamp(float(parts[1])).isoformat()[0:19] + "Z"
                    if rec["timestamp"].find('.') != -1:
                        raise RuntimeError()
                else:
                    rec[parts[0]] = parts[1]
            ret.append(rec)
        return ret

    def load_map_json(self):
        try:
            self.map_json = json.load(open(self.mapfilename))
        except:
            self.publog.error("Could not open json data {}".format(self.mapfilename))

    def mapfilerun(self, mountpoints=None):

        with open(self.mapfilename) as self.map_data:
            ret = self.parse_map(mountpoints)

        return ret



import os
from esgcet.create_ip import CreateIP
from esgcet.mkd_cmip5 import ESGPubMKDCmip5
from esgcet.settings import VARIABLE_LIMIT
import tempfile
import esgcet.logger as logger

log = logger.ESGPubLogger()


class cmip5(CreateIP):

    def __init__(self, argdict):
        super().__init__(argdict)
        self.variable_limit = 100
        self.autoc_args = ' --out_pretty --out_json {} --files "{}/*.nc"'
        self.publog = log.return_logger('CMIP5', self.silent, self.verbose)

    def autocurator(self, map_json_data):
        datafile = map_json_data[0][1]
        destpath = os.path.dirname(datafile)

        autstr = self.autoc_command + self.autoc_args
        files = os.listdir(destpath)
        for f in files:
            var = f.split('_')[0]
            if var not in self.variables:
                self.variables.append(var)
        for var in self.variables:
            self.scans.append(
                tempfile.NamedTemporaryFile())  # create a temporary file which is deleted afterward for autocurator
            scan = self.scans[-1].name
            self.publog.info("Autocurator command: " + autstr.format(scan, destpath))
            stat = os.system(autstr.format(scan, destpath))
            if os.WEXITSTATUS(stat) != 0:
                self.publog.error("Autocurator exited with exit code: " + str(os.WEXITSTATUS(stat)))
                self.cleanup()
                exit(os.WEXITSTATUS(stat))

    def mk_dataset(self, map_json_data):
        limit_exceeded = len(self.variables) > VARIABLE_LIMIT
        limit = False
        mkd = ESGPubMKDCmip5(self.data_node, self.index_node, self.replica, self.globus, self.data_roots,
                                self.dtn, self.silent, self.verbose, limit_exceeded)
        for scan in self.scans:
            try:
                out_json_data = mkd.get_records(map_json_data, scan.name, self.json_file)
                self.datasets.append(out_json_data)
            except:
                self.publog.exception("Occured while making dataset.")
                self.cleanup()
                exit(1)
            # only use first scan file if more than 75 variables
            if len(self.variables) > self.variable_limit:
                limit = True
                break

        self.master_dataset = mkd.aggregate_datasets(self.datasets, limit)
        return 0
